"""Tests for tvtk_base.py.  These only test the basic functionality of
the code in tvtk_base.  More tests for TVTK objects should be in
test_tvtk.py.

"""
# Author: Prabhu Ramachandran
# Copyright (c) 2004-2018, Enthought, Inc.
# License: BSD Style.

import unittest
import pickle
import weakref
import vtk
import gc
import imp

from traits import api as traits
from tvtk import tvtk_base
from tvtk.common import get_tvtk_name, camel2enthought


# An elementary class based on vtkProperty that is used only for
# testing.
class Prop(tvtk_base.TVTKBase):
    def __init__(self, obj=None, update=1, **traits):
        tvtk_base.TVTKBase.__init__(
            self, vtk.vtkProperty, obj, update, **traits
        )

    edge_visibility = tvtk_base.false_bool_trait

    def _edge_visibility_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetEdgeVisibility, self.edge_visibility_)

    representation = traits.Trait(
        'surface',
        tvtk_base.TraitRevPrefixMap(
            {'points': 0, 'wireframe': 1, 'surface': 2})
    )

    def _representation_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetRepresentation, self.representation_)

    opacity = traits.Trait(1.0, traits.Range(0.0, 1.0))

    def _opacity_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetOpacity,
                        self.opacity)

    specular_color = tvtk_base.vtk_color_trait((1.0, 1.0, 1.0))

    def _specular_color_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetSpecularColor,
                        self.specular_color, 1)

    diffuse_color = tvtk_base.vtk_color_trait((1.0, 1.0, 1.0))

    def _diffuse_color_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetDiffuseColor,
                        self.diffuse_color, 1)

    color = tvtk_base.vtk_color_trait((1.0, 1.0, 1.0))

    def _color_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetColor,
                        self.color)

    _updateable_traits_ = (('edge_visibility', 'GetEdgeVisibility'),
                           ('opacity', 'GetOpacity'),
                           ('specular_color', 'GetSpecularColor'),
                           ('color', 'GetColor'),
                           ('diffuse_color', 'GetDiffuseColor'),
                           ('representation', 'GetRepresentation'))


class TestTVTKBase(unittest.TestCase):
    def test_tvtk_name(self):
        """Test VTK to TVTK class name conversion."""
        v_name = ['vtkFooBar', 'vtkXMLDataReader',
                  'vtk3DSReader', 'vtk2000Bug']
        t_name = ['FooBar', 'XMLDataReader',
                  'ThreeDSReader', 'Two000Bug']
        for i, vn in enumerate(v_name):
            tn = get_tvtk_name(vn)
            self.assertEqual(tn, t_name[i])

        num = ['Zero', 'One', 'Two', 'Three', 'Four', 'Five',
               'Six', 'Seven', 'Eight', 'Nine']
        for i in range(10):
            vn = 'vtk%dA' % i
            tn = get_tvtk_name(vn)
            self.assertEqual(tn, '%sA' % num[i])

    def test_camel2enthought(self):
        """Test CamelCase to Enthought style name conversion."""
        v_name = ['GetFooBar', 'GetOBBTree', 'XMLDataReader',
                  'GetFooXML', 'HTMLIsSGML', '_SetMe', '_XYZTest',
                  'Actor2D', 'Actor3D', 'Actor6D', 'PLOT3DReader',
                  'Actor61Dimension', 'GL2PSExporter',
                  'Volume16Reader']
        t_name = ['get_foo_bar', 'get_obb_tree', 'xml_data_reader',
                  'get_foo_xml', 'html_is_sgml', '_set_me',
                  '_xyz_test', 'actor2d', 'actor3d', 'actor6_d',
                  'plot3d_reader', 'actor61_dimension',
                  'gl2ps_exporter', 'volume16_reader']
        for i, vn in enumerate(v_name):
            tn = camel2enthought(vn)
            self.assertEqual(tn, t_name[i])

    def test_do_change(self):
        """Test if VTK object changes when trait is changed."""
        p = Prop()
        p.edge_visibility = not p.edge_visibility
        p.representation = 'p'
        p.opacity = 0.5
        p.color = (0, 1, 0)
        p.diffuse_color = (1, 1, 1)
        p.specular_color = (1, 1, 0)
        for t, g in p._updateable_traits_:
            val = getattr(p._vtk_obj, g)()
            if t == 'representation':
                self.assertEqual(val, getattr(p, t + '_'))
            else:
                self.assertEqual(val, getattr(p, t))

    def test_wrap_call_is_graceful_on_failure(self):
        # Given
        p = Prop()

        # When
        try:
            # Make a mistake
            p._wrap_call(p._vtk_obj.SetLineWidth, 'a')
        except TypeError:
            pass

        # Then
        # The _in_set should be reset to zero.
        self.assertEqual(p._in_set, 0)

    def test_auto_update(self):
        """Test trait updation when the VTK object changes."""
        p = Prop()
        obj = p._vtk_obj
        obj.SetEdgeVisibility(1)
        self.assertEqual(p.edge_visibility, 1)

        obj.SetOpacity(0.5)
        self.assertEqual(p.opacity, 0.5)

        obj.SetRepresentationToPoints()
        self.assertEqual(p.representation, 'points')

        val = (1.0, 1.0, 0.0)
        obj.SetColor(val)
        self.assertEqual(p.color, val)

        val = (1.0, 0.0, 0.0)
        obj.SetDiffuseColor(val)
        self.assertEqual(p.diffuse_color, val)

        val = (0.0, 1.0, 0.0)
        obj.SetSpecularColor(val)
        self.assertEqual(p.specular_color, val)

    def test_setup_teardown_observers(self):
        """If setup_observers and teardown_observers work correctly."""
        p = Prop()
        # Turn off the observers.
        p.teardown_observers()
        obj = p._vtk_obj
        obj.SetEdgeVisibility(1)
        self.assertEqual(p.edge_visibility, 0)

        obj.SetOpacity(0.5)
        self.assertEqual(p.opacity, 1.0)

        obj.SetRepresentationToPoints()
        self.assertEqual(p.representation, 'surface')

        # Setup the observers again.
        p.update_traits()
        p.setup_observers()

        self.assertEqual(p.edge_visibility, 1)
        self.assertEqual(p.opacity, 0.5)
        self.assertEqual(p.representation, 'points')

        obj.SetEdgeVisibility(0)
        self.assertEqual(p.edge_visibility, 0)

        obj.SetOpacity(1.0)
        self.assertEqual(p.opacity, 1.0)

        obj.SetRepresentationToSurface()
        self.assertEqual(p.representation, 'surface')

    def test_pickle(self):
        """Test if pickling works."""
        p = Prop()
        p.edge_visibility = 1
        p.representation = 'p'
        p.opacity = 0.5
        p.color = (0, 1, 0)
        p.diffuse_color = (0, 1, 1)
        p.specular_color = (1, 1, 0)

        s = pickle.dumps(p)
        del p

        p = pickle.loads(s)

        self.assertEqual(p.edge_visibility, 1)
        self.assertEqual(p.opacity, 0.5)
        self.assertEqual(p.representation, 'points')
        val = (0.0, 1.0, 1.0)
        self.assertEqual(p.color, val)
        val = (0.0, 1.0, 1.0)
        self.assertEqual(p.diffuse_color, val)
        val = (1.0, 1.0, 0.0)
        self.assertEqual(p.specular_color, val)

        # test if pickling also works on an existing object.
        d = p.__getstate__()
        del p
        p = Prop()
        addr = p._vtk_obj.__this__
        p.__setstate__(d)
        # Make sure its the same object.
        self.assertEqual(addr, p._vtk_obj.__this__)

        self.assertEqual(p.edge_visibility, 1)
        self.assertEqual(p.opacity, 0.5)
        self.assertEqual(p.representation, 'points')
        val = (0.0, 1.0, 1.0)
        self.assertEqual(p.color, val)

    def test_rev_prefix_map(self):
        """Test the reverse prefix map trait we use."""
        p = Prop()
        p.representation = 'p'
        p.representation = 'wire'
        p.representation = 'points'
        p.representation = 2
        self.assertEqual(p.representation, 'surface')
        self.assertRaises(traits.TraitError, setattr, p,
                          'representation', 'points1')
        self.assertRaises(traits.TraitError, setattr, p,
                          'representation', 'POINTS')

    def test_deref_vtk(self):
        """Test the `deref_vtk` function."""
        p = Prop()
        o = tvtk_base.deref_vtk(p)
        self.assertEqual(o.IsA('vtkProperty'), True)
        o1 = tvtk_base.deref_vtk(o)
        self.assertEqual(o1.IsA('vtkProperty'), True)

        o = tvtk_base.deref_vtk(self)
        self.assertEqual(o.__class__.__name__, 'TestTVTKBase')

    def test_obj_del(self):
        """Test object deletion and reference cycles."""
        p = Prop()
        p.representation = 0
        ref = weakref.ref(p)
        del p
        self.assertEqual(ref(), None)

    def test_global_disable_update(self):
        # Given
        p = Prop()
        vp = tvtk_base.deref_vtk(p)

        # When
        with tvtk_base.global_disable_update():
            vp.SetOpacity(0.5)
            vp.Modified()

        # Then
        self.assertEqual(p.opacity, 1.0)

        # When
        vp.SetOpacity(0.4)

        # Then
        self.assertEqual(p.opacity, 0.4)

    def test_strict_traits(self):
        """Test if TVTK objects use strict traits."""
        p = Prop()
        self.assertRaises(traits.TraitError, setattr, p, 'foo', 1)
        self.assertRaises(traits.TraitError, setattr, p, '_foo', 1)
        self.assertRaises(traits.TraitError, setattr, p, '__foo', 1)

    def test_init_traits(self):
        """Test if the objects traits can be set in __init__."""
        p = Prop(opacity=0.1, color=(1, 0, 0), representation='p')
        self.assertEqual(p.opacity, 0.1)
        self.assertEqual(p.color, (1.0, 0.0, 0.0))
        self.assertEqual(p.representation, 'points')

        # Test case where the object traits are wrong.
        self.assertRaises(traits.TraitError, Prop, foo='bar')

    def test_zz_object_cache(self):
        """Test if object cache works correctly."""
        # HACK!  The zz in the method name ensures that this is run
        # last.  The reloading messes up some of the other tests
        # because the Prop classes base class is different.

        l1 = len(tvtk_base._object_cache)
        p = Prop()
        addr = p._vtk_obj.__this__
        self.assertEqual(l1 + 1, len(tvtk_base._object_cache))
        self.assertEqual(p, tvtk_base._object_cache.get(addr))

        del p
        gc.collect()  # Force collection.
        self.assertEqual(l1, len(tvtk_base._object_cache))
        self.assertEqual(addr in tvtk_base._object_cache, False)

        # Check reload-safety.
        p = Prop()
        l1 = len(tvtk_base._object_cache)
        imp.reload(tvtk_base)
        self.assertEqual(l1, len(tvtk_base._object_cache))

    # Reloading causes havoc with nosetests based tests so we skip in
    # that case.  Unittest will see the test just fine.
    test_zz_object_cache.__test__ = False


if __name__ == "__main__":
    unittest.main()



"""
Metadata for all modules.
"""
# Author: Prabhu Ramachandran <prabhu@aero.iitb.ac.in>
# Copyright (c) 2008, Prabhu Ramachandran Enthought, Inc.
# License: BSD Style.

# Local imports.
from mayavi.core.metadata import ModuleMetadata
from mayavi.core.pipeline_info import PipelineInfo

BASE = 'mayavi.modules'


################################################################################
# Metadata.

axes_module = ModuleMetadata(
    id            = "AxesModule",
    menu_name          = "&Axes",
    class_name = BASE + '.axes.Axes',
    desc   = "Draw cubical axes on the outline for given input",
    tooltip       = "Draw axes on the outline of input data",
    help       = "Draw axes on the outline of input data",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

contour_grid_plane_module = ModuleMetadata(
    id            = "ContourGridPlaneModule",
    menu_name          = "&ContourGridPlane",
    class_name = BASE + '.contour_grid_plane.ContourGridPlane',
    desc   = "Shows a contour grid plane for the given input",
    tooltip       = "Shows a contour grid plane for the given input",
    help       = "Shows a contour grid plane for the given input",
    input_info = PipelineInfo(datasets=['image_data',
                                        'structured_grid',
                                        'rectilinear_grid'],
                              attribute_types=['any'],
                              attributes=['any'])
)

custom_grid_plane_module = ModuleMetadata(
    id            = "CustomGridPlaneModule",
    menu_name          = "CustomGridPlane",
    class_name = BASE + '.custom_grid_plane.CustomGridPlane',
    desc   = "Creates a highly customizable grid plane for given input",
    tooltip       = "Creates a highly customizable grid plane for given input",
    help       = "Creates a highly customizable grid plane for given input",
    input_info = PipelineInfo(datasets=['image_data',
                                        'structured_grid',
                                        'rectilinear_grid'],
                              attribute_types=['any'],
                              attributes=['any'])
)

glyph_module = ModuleMetadata(
    id            = "GlyphModule",
    menu_name          = "Gl&yph",
    class_name = BASE + '.glyph.Glyph',
    desc   = "Creates colored and scaled glyphs at at input points",
    tooltip       = "Creates colored and scaled glyphs at at input points",
    help       = "Creates colored and scaled glyphs at at input points",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

grid_plane_module = ModuleMetadata(
    id            = "GridPlaneModule",
    menu_name          = "&GridPlane",
    class_name = BASE + '.grid_plane.GridPlane',
    desc   = "Shows a grid plane for the given input",
    tooltip       = "Shows a grid plane for the given input",
    help       = "Shows a grid plane for the given input",
    input_info = PipelineInfo(datasets=['image_data',
                                        'structured_grid',
                                        'rectilinear_grid'],
                              attribute_types=['any'],
                              attributes=['any'])
)

hyper_streamline_module = ModuleMetadata(
    id            = "HyperStreamlineModule",
    menu_name          = "&HyperStreamline",
    class_name = BASE + '.hyper_streamline.HyperStreamline',
    desc   = "Shows hyper streamlines for tensor data",
    tooltip       = "Shows hyper streamlines for tensor data",
    help       = "Shows hyper streamlines for tensor data",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['tensors'])
)

image_actor_module = ModuleMetadata(
    id            = "ImageActorModule",
    menu_name          = "&ImageActor",
    class_name = BASE + '.image_actor.ImageActor',
    desc   = "Shows an image actor for image data",
    tooltip       = "Shows an image actor for image data",
    help       = "Shows an image actor for image data",
    input_info = PipelineInfo(datasets=['image_data'],
                              attribute_types=['any'],
                              attributes=['any'])
)

image_plane_widget_module = ModuleMetadata(
    id            = "ImagePlaneWidgetModule",
    menu_name          = "I&magePlaneWidget",
    class_name = BASE + '.image_plane_widget.ImagePlaneWidget',
    desc   = "Shows an image plane widget for image data",
    tooltip       = "Shows an image plane widget for image data",
    help       = "Shows an image plane widget for image data",
    input_info = PipelineInfo(datasets=['image_data'],
                              attribute_types=['any'],
                              attributes=['scalars'])
)

isosurface_module = ModuleMetadata(
    id            = "IsoSurfaceModule",
    menu_name          = "&IsoSurface",
    class_name = BASE + '.iso_surface.IsoSurface',
    desc   = "Creates an iso-surface for the given input",
    tooltip       = "Creates an iso-surface for the given input",
    help       = "Creates an iso-surface for the given input",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['scalars'])
)

labels_module = ModuleMetadata(
    id            = "LabelsModule",
    menu_name          = "&Labels",
    class_name = BASE + '.labels.Labels',
    desc   = "Display labels for active dataset or active module",
    tooltip       = "Display labels for active dataset or active module",
    help       = "Display labels for active dataset or active module",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

orientation_axes_module = ModuleMetadata(
    id            = "OrientationAxesModule",
    menu_name          = "Orientation A&xes",
    class_name = BASE + '.orientation_axes.OrientationAxes',
    desc   = "Show an axes indicating the current orientation",
    tooltip       = "Show an axes indicating the current orientation",
    help       = "Show an axes indicating the current orientation",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

outline_module = ModuleMetadata(
    id            = "OutlineModule",
    menu_name          = "&Outline",
    class_name = BASE + '.outline.Outline',
    desc   = "Draw an outline for given input",
    tooltip       = "Draw an outline for given input",
    help       = "Draw an outline for given input",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

scalar_cut_plane_module = ModuleMetadata(
    id            = "ScalarCutPlaneModule",
    menu_name          = "Scalar Cut &Plane",
    class_name = BASE + '.scalar_cut_plane.ScalarCutPlane',
    desc   = "Slice through the data with optional contours",
    tooltip       = "Slice through the data with optional contours",
    help       = "Slice through the data with optional contours",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['scalars'])
)

slice_ug_module = ModuleMetadata(
    id            = "SliceUnstructuredGridModule",
    menu_name          = "Slice &Unstructured Grid",
    class_name = BASE + '.slice_unstructured_grid.SliceUnstructuredGrid',
    desc   = "Slice an unstructured grid to show cells",
    tooltip       = "Slice an unstructured grid to show cells",
    help       = "Slice an unstructured grid to show cells",
    input_info = PipelineInfo(datasets=['unstructured_grid'],
                              attribute_types=['any'],
                              attributes=['any'])
)

sgrid_outline_module = ModuleMetadata(
    id            = "StructuredGridOutlineModule",
    menu_name          = "StructuredGridOutline",
    class_name = BASE + '.structured_grid_outline.StructuredGridOutline',
    desc   = "Draw a grid-conforming outline for structured grids",
    tooltip       = "Draw a grid-conforming outline for structured grids",
    help       = "Draw a grid-conforming outline for structured grids",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

streamline_module = ModuleMetadata(
    id            = "StreamlineModule",
    menu_name          = "Stream&line",
    class_name = BASE + '.streamline.Streamline',
    desc   = "Generate streamlines for the vectors",
    tooltip       = "Generate streamlines for the vectors",
    help       = "Generate streamlines for the vectors",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['vectors'])
)

surface_module = ModuleMetadata(
    id            = "SurfaceModule",
    menu_name          = "&Surface",
    class_name = BASE + '.surface.Surface',
    desc   = "Creates a surface for the given input",
    tooltip       = "Creates a surface for the given input",
    help       = "Creates a surface for the given input",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

tensor_glyph_module = ModuleMetadata(
    id            = "TensorGlyphModule",
    menu_name          = "Te&nsorGlyph",
    class_name = BASE + '.tensor_glyph.TensorGlyph',
    desc   = "Displays glyphs scaled and oriented as per tensor data",
    tooltip       = "Displays glyphs scaled and oriented as per tensor data",
    help       = "Displays glyphs scaled and oriented as per tensor data",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['tensors'])
)

text_module = ModuleMetadata(
    id            = "TextModule",
    menu_name          = "&Text",
    class_name = BASE + '.text.Text',
    desc   = "Displays user specified text on screen",
    tooltip       = "Displays text on screen",
    help       = "Displays text on screen",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

text3d_module = ModuleMetadata(
    id            = "Text3DModule",
    menu_name          = "&Text3D",
    class_name = BASE + '.text3d.Text3D',
    desc   = "Displays user-specified text at a 3D location in the scene",
    tooltip    = "Displays user-specified text at a 3D location in the scene",
    help       = "Displays user-specified text at a 3D location in the scene",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

vector_cut_plane_module = ModuleMetadata(
    id            = "VectorCutPlaneModule",
    menu_name          = "&VectorCutPlane",
    class_name = BASE + '.vector_cut_plane.VectorCutPlane',
    desc   = "Display vectors along a cut plane",
    tooltip       = "Display vectors along a cut plane",
    help       = "Display vectors along a cut plane",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['vectors'])
)

vectors_module = ModuleMetadata(
    id            = "VectorsModule",
    menu_name          = "Vecto&rs",
    class_name = BASE + '.vectors.Vectors',
    desc   = "Display input vectors using arrows or other glyphs",
    tooltip       = "Display input vectors using arrows or other glyphs",
    help       = "Display input vectors using arrows or other glyphs",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['any'])
)

volume_module = ModuleMetadata(
    id            = "VolumeModule",
    menu_name          = "Volum&e",
    class_name = BASE + '.volume.Volume',
    desc   = "Use volume rendering to view the scalar field",
    tooltip       = "Use volume rendering to view the scalar field",
    help       = "Use volume rendering to view the scalar field",
    input_info = PipelineInfo(datasets=['image_data',
                                        'unstructured_grid'],
                              attribute_types=['any'],
                              attributes=['scalars'])
)

warp_vector_cut_plane_module = ModuleMetadata(
    id            = "WarpVectorCutPlaneModule",
    menu_name          = "&WarpVectorCutPlane",
    class_name = BASE + '.warp_vector_cut_plane.WarpVectorCutPlane',
    desc   = "Warp cut plane along scaled input vectors",
    tooltip       = "Warp cut plane along scaled input vectors",
    help       = "Warp cut plane along scaled input vectors",
    input_info = PipelineInfo(datasets=['any'],
                              attribute_types=['any'],
                              attributes=['vectors'])
)


# Now collect all the modules for the mayavi registry.
modules = [axes_module,
           contour_grid_plane_module,
           custom_grid_plane_module,
           glyph_module,
           grid_plane_module,
           hyper_streamline_module,
           image_actor_module,
           image_plane_widget_module,
           isosurface_module,
           labels_module,
           orientation_axes_module,
           outline_module,
           scalar_cut_plane_module,
           slice_ug_module,
           sgrid_outline_module,
           streamline_module,
           surface_module,
           tensor_glyph_module,
           text_module,
           text3d_module,
           vector_cut_plane_module,
           vectors_module,
           volume_module,
           warp_vector_cut_plane_module,
          ]




"""A component that allows one to place colored and scaled glyphs at
input point data.
"""
# Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
#         KK Rai (kk.rai [at] iitb.ac.in)
#         R. Ambareesha (ambareesha [at] iitb.ac.in)
# Copyright (c) 2005-2018, Enthought, Inc.
# License: BSD Style.

# Enthought library imports.
from traits.api import Instance, Trait, Bool
from traits.api import Enum
from traitsui.api import View, Group, Item
from tvtk.api import tvtk
from tvtk.tvtk_base import TraitRevPrefixMap

# Local imports.
from mayavi.core.component import Component
from mayavi.core.module import Module
from mayavi.components import glyph_source


######################################################################
# `Glyph` class.
######################################################################
class Glyph(Component):

    # The version of this class.  Used for persistence.
    __version__ = 0

    # Type of Glyph: 'tensor' or 'vector'
    glyph_type = Enum('vector', 'tensor',
                      desc = 'if the glyph is vector or tensor')

    # The scaling mode to use when scaling the glyphs.  We could have
    # used the glyph's own scale mode but it allows users to set the
    # mode to use vector components for the scaling which I'd like to
    # disallow.
    scale_mode = Trait('scale_by_scalar',
                       TraitRevPrefixMap({'scale_by_vector': 1,
                                          'scale_by_vector_components': 2,
                                          'data_scaling_off': 3,
                                          'scale_by_scalar': 0}),
                       desc="if scaling is done using scalar or vector/normal magnitude"
                       )

    # The color mode to use when coloring the glyphs.  We could have
    # used the glyph's own color_mode trait but it allows users to set
    # the mode to use vector components for the scaling which I'd
    # like to disallow.
    color_mode = Trait('color_by_scalar',
                       TraitRevPrefixMap({'color_by_vector': 2,
                                          'color_by_scalar': 1,
                                          'no_coloring': 0}),
                       desc="if coloring is done by scalar or vector/normal magnitude"
                       )
    color_mode_tensor = Trait('scalar',
                              TraitRevPrefixMap({'scalars': 1,
                                                 'eigenvalues':2,
                                                 'no_coloring': 0}),
                              desc="if coloring is done by scalar or eigenvalues"
                             )

    # Specify if the input points must be masked.  By mask we mean
    # that only a subset of the input points must be displayed.
    mask_input_points = Bool(False, desc="if input points are masked")

    # The MaskPoints filter.
    mask_points = Instance(tvtk.MaskPoints, args=(),
                           kw={'random_mode': True}, record=True)

    # The Glyph3D instance.
    glyph = Instance(tvtk.Object, allow_none=False, record=True)

    # The Source to use for the glyph.  This is chosen from
    # `self._glyph_list` or `self.glyph_dict`.
    glyph_source = Instance(glyph_source.GlyphSource,
                            allow_none=False, record=True)

    # The module associated with this component.  This is used to get
    # the data range of the glyph when the scale mode changes.  This
    # *must* be set if this module is to work correctly.
    module = Instance(Module)

    # Should we show the GUI option for changing the scalar mode or
    # not?  This is useful for vector glyphing modules where there it
    # does not make sense to scale the data based on scalars.
    show_scale_mode = Bool(True)

    ########################################
    # Private traits.

    # Used for optimization.
    _updating = Bool(False)

    ########################################
    # View related traits.

    view = View(Group(Item(name='mask_input_points'),
                      Group(Item(name='mask_points',
                                 enabled_when='object.mask_input_points',
                                 style='custom', resizable=True),
                            show_labels=False,
                            ),
                      label='Masking',
                      ),
                Group(Group(Item(name='scale_mode',
                                 enabled_when='show_scale_mode',
                                 visible_when='show_scale_mode'),
                            Item(name='color_mode',
                                 enabled_when= 'glyph_type == "vector"',
                                 visible_when= 'glyph_type == "vector"'),
                            Item(name='color_mode_tensor',
                                 enabled_when= 'glyph_type == "tensor"',
                                 visible_when= 'glyph_type == "tensor"'),
                            ),
                      Group(Item(name='glyph', style='custom',
                                 resizable=True),
                            show_labels=False),
                      label='Glyph',
                      selected=True,
                      ),
                Group(Item(name='glyph_source',
                                 style='custom', resizable=True),
                      show_labels=False,
                      label='Glyph Source',
                      ),
                resizable=True
                )

    ######################################################################
    # `object` interface
    ######################################################################
    def __get_pure_state__(self):
        d = super(Glyph, self).__get_pure_state__()
        for attr in ('module', '_updating'):
            d.pop(attr, None)
        return d


    ######################################################################
    # `Module` interface
    ######################################################################
    def setup_pipeline(self):
        """Override this method so that it *creates* the tvtk
        pipeline.

        This method is invoked when the object is initialized via
        `__init__`.  Note that at the time this method is called, the
        tvtk data pipeline will *not* yet be setup.  So upstream data
        will not be available.  The idea is that you simply create the
        basic objects and setup those parts of the pipeline not
        dependent on upstream sources and filters.  You should also
        set the `actors` attribute up at this point.
        """
        self._glyph_type_changed(self.glyph_type)
        self.glyph_source = glyph_source.GlyphSource()

        # Handlers to setup our source when the sources pipeline changes.
        self.glyph_source.on_trait_change(self._update_source, 'pipeline_changed')
        self.mask_points.on_trait_change(self.render)

    def update_pipeline(self):
        """Override this method so that it *updates* the tvtk pipeline
        when data upstream is known to have changed.

        This method is invoked (automatically) when any of the inputs
        sends a `pipeline_changed` event.
        """
        if ((len(self.inputs) == 0) or (len(self.inputs[0].outputs) == 0)):
            return

        self._mask_input_points_changed(self.mask_input_points)
        if self.glyph_type == 'vector':
            self._color_mode_changed(self.color_mode)
        else:
            self._color_mode_tensor_changed(self.color_mode_tensor)
        self._scale_mode_changed(self.scale_mode)

        # Set our output.
        self.outputs = [self.glyph]
        self.pipeline_changed = True

    def update_data(self):
        """Override this method so that it flushes the vtk pipeline if
        that is necessary.

        This method is invoked (automatically) when any of the inputs
        sends a `data_changed` event.
        """
        self._scale_mode_changed(self.scale_mode)
        self.data_changed = True

    def render(self):
        if not self._updating:
            super(Glyph, self).render()

    def start(self):
        """Overridden method.
        """
        if self.running:
            return
        self.glyph_source.start()
        super(Glyph, self).start()

    def stop(self):
        if not self.running:
            return
        self.glyph_source.stop()
        super(Glyph, self).stop()

    def has_output_port(self):
        """ The filter has an output port."""
        return True

    def get_output_object(self):
        """ Returns the output port."""
        return self.glyph.output_port

    ######################################################################
    # Non-public methods.
    ######################################################################
    def _update_source(self):
        self.configure_source_data(self.glyph, self.glyph_source.outputs[0])

    def _glyph_source_changed(self, value):
        self.configure_source_data(self.glyph, value.outputs[0])

    def _color_mode_changed(self, value):
        if len(self.inputs) == 0:
            return
        if value != 'no_coloring':
            self.glyph.color_mode = value

    def _color_mode_tensor_changed(self, value):
        if len(self.inputs) == 0:
            return
        self._updating = True
        if value != 'no_coloring':
            self.glyph.color_mode = value
            self.glyph.color_glyphs = True
        else:
            self.glyph.color_glyphs = False
        self._updating = False
        self.render()

    def _scale_mode_changed(self, value):
        if (self.module is None) or (len(self.inputs) == 0)\
                                 or self.glyph_type == 'tensor':
            return

        self._updating = True
        try:
            glyph = self.glyph
            glyph.scale_mode = value

            mm = self.module.module_manager
            if glyph.scale_mode == 'scale_by_scalar':
                glyph.range = tuple(mm.scalar_lut_manager.data_range)
            else:
                glyph.range = tuple(mm.vector_lut_manager.data_range)
        finally:
            self._updating = False
            self.render()

    def _mask_input_points_changed(self, value):
        inputs = self.inputs
        if len(inputs) == 0:
            return
        if value:
            mask = self.mask_points
            self.configure_connection(mask, inputs[0].outputs[0])
            self.configure_connection(self.glyph, mask)
        else:
            self.configure_connection(self.glyph, inputs[0])
        self.glyph.update()

    def _glyph_type_changed(self, value):
        if self.glyph_type == 'vector':
            self.glyph = tvtk.Glyph3D(clamping=True)
        else:
            self.glyph = tvtk.TensorGlyph(scale_factor=0.1)
            self.show_scale_mode = False
        self.glyph.on_trait_change(self.render)

    def _scene_changed(self, old, new):
        super(Glyph, self)._scene_changed(old, new)
        self.glyph_source.scene = new



"""
Collection of utility functions common to different toolkits.
"""

def popup_save(parent=None):
    """Popup a dialog asking for an image name to save the scene to.
    This is used mainly to save a scene in full screen mode. Returns a
    filename, returns empty string if action was cancelled. `parent` is
    the parent widget over which the dialog will be popped up.
    """
    from pyface.api import FileDialog, OK

    extensions = ['*.png', '*.jpg', '*.tiff', '*.bmp', '*.ps',
                  '*.eps', '*.pdf', '*.tex', '*.rib', '*.wrl',
                  '*.oogl', '*.vrml', '*.obj', '*.iv', '*.pov',
                  '*.x3d']
    descriptions = ["PNG", "JPG", "TIFF", "Bitmap", "PostScript",
                    "EPS", "PDF", "Tex", "RIB", "WRL",
                    "Geomview", "VRML", "Wavefront", "Open Inventor",
                    "Povray", "X3D"]
    wildcard = ""
    for description, extension in zip(descriptions, extensions):
        wildcard += "{} ({})|{}|".format(description,
                                         extension,
                                         extension)
    wildcard += "Determine by extension (*.*)|(*.*)"

    dialog = FileDialog(
        parent=parent, title='Save scene to image',
        action='save as', default_filename="snapshot.png",
        wildcard=wildcard
    )
    if dialog.open() == OK:
        return dialog.path
    else:
        return ''



"""The  Optional filter is one which may be turned on and off and wraps
around any mayavi filter or component.
"""
# Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
# Copyright (c) 2008,  Prabhu Ramachandran
# License: BSD Style.

from mayavi.filters.wrapper import Wrapper

################################################################################
# `Optional` class.
################################################################################
class Optional(Wrapper):
    """
    This class wraps around any mayavi filter or component and allows a
    user to turn it on or off.
    """

    # This filter should allow us to turn on/off the filter.
    _show_enabled = True




"""This component computes normals for input poly data.
"""
# Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
# Copyright (c) 2005-2016,  Enthought, Inc.
# License: BSD Style.

# Enthought library imports.
from traits.api import Instance
from traitsui.api import View, Group, Item, InstanceEditor
from tvtk.api import tvtk

# Local imports.
from mayavi.core.component import Component
from mayavi.components.common import convert_to_poly_data


######################################################################
# `PolyDataNormals` class.
######################################################################
class PolyDataNormals(Component):
    # The version of this class.  Used for persistence.
    __version__ = 0

    # The filter that generates the normals.
    filter = Instance(tvtk.PolyDataNormals, args=(),
                      kw={'feature_angle': 45.0}, record=True)

    ########################################
    # The component's view

    _filter_group = Group(Item(name='feature_angle'))

    view = View(Group(Item(name='filter', style='custom',
                           editor=InstanceEditor(view=View(_filter_group))),
                      show_labels=False
                      )
                )

    ######################################################################
    # `Component` interface
    ######################################################################
    def setup_pipeline(self):
        """Override this method so that it *creates* its tvtk
        pipeline.

        This method is invoked when the object is initialized via
        `__init__`.  Note that at the time this method is called, the
        tvtk data pipeline will *not* yet be setup.  So upstream data
        will not be available.  The idea is that you simply create the
        basic objects and setup those parts of the pipeline not
        dependent on upstream sources and filters.
        """
        self.filter.on_trait_change(self.update_data)

    def update_pipeline(self):
        """Override this method so that it *updates* the tvtk pipeline
        when data upstream is known to have changed.

        This method is invoked (automatically) when the input fires a
        `pipeline_changed` event.
        """
        if (len(self.inputs) == 0) or \
           (len(self.inputs[0].outputs) == 0):
            return
        f = self.filter
        input = self.inputs[0].outputs[0]
        self.configure_input(f, convert_to_poly_data(input))
        f.update()
        self.outputs = [f]

    def update_data(self):
        """Override this method to do what is necessary when upstream
        data changes.

        This method is invoked (automatically) when any of the inputs
        sends a `data_changed` event.
        """
        self.data_changed = True

    def has_output_port(self):
        """ The filter has an output port."""
        return True

    def get_output_object(self):
        """ Returns the output port."""
        return self.filter.output_port



# Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
# Copyright (c) 2008, Enthought, Inc.
# License: BSD Style.

# Enthought library imports.
from traits.api import Instance
from tvtk.api import tvtk

# Local imports
from mayavi.filters.filter_base import FilterBase
from mayavi.core.pipeline_info import PipelineInfo


######################################################################
# `Stripper` class.
######################################################################
class Stripper(FilterBase):

    """ Create triangle strips and/or poly-lines. Useful for regularizing
        broken up surfaces, such as those created by the Tube
        filter.
    """

    # The version of this class.  Used for persistence.
    __version__ = 0

    # The actual TVTK filter that this class manages.
    filter = Instance(tvtk.Stripper, args=(), allow_none=False, record=True)

    input_info = PipelineInfo(datasets=['poly_data'],
                              attribute_types=['any'],
                              attributes=['any'])

    output_info = PipelineInfo(datasets=['poly_data'],
                               attribute_types=['any'],
                               attributes=['any'])




"""Module documentation goes here."""

# Enthought library imports.
from traits.api import Instance
from traitsui.api import View, Group, Item
from tvtk.api import tvtk

# Local imports
from mayavi.core.module import Module
from mayavi.components.actor import Actor


######################################################################
# `MyModule` class.
######################################################################
class MyModule(Module):

    # The version of this class.  Used for persistence.
    __version__ = 0

    # The actor component that represents the visualization.
    actor = Instance(Actor)

    ########################################
    # View related code.


    ######################################################################
    # `Module` interface
    ######################################################################
    def setup_pipeline(self):
        """Override this method so that it *creates* the tvtk
        pipeline.

        This method is invoked when the object is initialized via
        `__init__`.  Note that at the time this method is called, the
        tvtk data pipeline will *not* yet be setup.  So upstream data
        will not be available.  The idea is that you simply create the
        basic objects and setup those parts of the pipeline not
        dependent on upstream sources and filters.  You should also
        set the `actors` attribute up at this point.
        """
        # Create the components and set them up.

        # Setup the actor suitably for this module.

        # Setup the components, actors and widgets. (sample code)
        #self.components.extend([your_components, ...])
        #self.actors.append(your_actor)
        # Note that self.actor.actor need not be added.
        #self.widgets.append(your_widget)

    def update_pipeline(self):
        """Override this method so that it *updates* the tvtk pipeline
        when data upstream is known to have changed.

        This method is invoked (automatically) when any of the inputs
        sends a `pipeline_changed` event.
        """
        # Data is available, so set the input for the grid plane.
        # Do your stuff here!

        # Now flush the pipeline
        self.pipeline_changed = True

    def update_data(self):
        """Override this method so that it flushes the vtk pipeline if
        that is necessary.

        This method is invoked (automatically) when any of the inputs
        sends a `data_changed` event.
        """
        # Just set data_changed, the components should do the rest if
        # they are connected.
        self.data_changed = True

    ######################################################################
    # Non-public methods.
    ######################################################################



from mayavi.version import version, version as __version__
from mayavi.core.engine import Engine
from mayavi.core.off_screen_engine import OffScreenEngine
from mayavi.tests.runtests import m2_tests as test



#!/usr/bin/env python

"""Script to run all the tests each in its own subprocess to ensure
maximal correctness and minimal pain.  This does make running the tests
quite slow but the tests will be run correctly -- which is the important
thing.
"""

# Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
# Copyright (c) 2009-2015,  Enthought Inc.
# License: BSD Style.

from __future__ import print_function

import sys
from os.path import dirname, join, isfile, isdir
from glob import glob
import subprocess
import time
import re
import optparse

# Use nosetests when it is available
USE_NOSE = False

def get_test_runner():
    """Get a test runner for the tests.  Uses nose if available."""
    result = [sys.executable]
    if USE_NOSE:
        try:
            import nose
        except ImportError:
            result = [sys.executable]
        else:
            result = ['nosetests']
    return result

def get_tests_in_dir(pth):
    """Get all tests in given directory `pth`."""
    files = []
    files.extend(glob(join(pth, 'test*.py')))
    files.extend(glob(join(pth, '*_test_case.py')))
    files.extend(glob(join(pth, 'test', 'test*.py')))
    files.extend(glob(join(pth, 'test', '*_test_case.py')))
    files.extend(glob(join(pth, 'tests', 'test*.py')))
    files.extend(glob(join(pth, 'tests', '*_test_case.py')))
    return files

def find_tests(tests):
    """Find test files given list of arguments which may be files,
    directories or modules."""
    files = []
    for test in tests:
        if isfile(test):
            files.append(test)
        elif isdir(test):
            files.extend(get_tests_in_dir(test))
        else:
            # A module.
            try:
                # Import the module
                components = test.split('.')
                if len(components) > 1:
                    modname = '.'.join(components[:-1])
                    symbol = components[-1]
                    mod = __import__(modname, globals(), locals(), [symbol])
                    s = getattr(mod, symbol)
                    d = dirname(s.__file__)
                else:
                    modname = components[0]
                    mod = __import__(modname, globals(), locals(), [])
                    d = dirname(mod.__file__)

                files.extend(get_tests_in_dir(d))
            except ImportError:
                msg = 'Warning: %s is neither a file/directory or '\
                      'module. Ignoring.'%test
                print(msg)
    return files

def run(tests, verbose=1):
    """Run the given tests.  Each test file is run as a unittest in a
    subprocess.

    **Parameters**

     :tests: List of test file paths to run as tests.  These tests are
             best written as unittests.

     :verbose: An integer in (0, 1, 2).  0 specifies no output.  1
               specifies moderate output and 2 very verbose output.

    **Returns**

    The error code is returned.  If there are errors this is 1 if
    everything ran fine it returns 0.

    """
    PIPE = subprocess.PIPE
    errors = []
    total = 0
    tot_err = 0
    tot_fail = 0
    total_time = 0.0
    cmd_base = get_test_runner()
    has_nose = 'nosetests' in cmd_base
    for test in tests:
        if verbose > 1 and has_nose:
            cmd = cmd_base + ['-v', test]
        else:
            cmd = cmd_base + [test]

        # Run the test in a subprocess.
        if verbose > 1:
            print('Running:', ' '.join(cmd))
        t1 = time.time()
        pipe = subprocess.Popen(cmd, stdout=PIPE, stderr=PIPE, close_fds=True)
        # Get the status, stdout and stderr.
        st = pipe.wait()
        t2 = time.time()
        out, err = pipe.communicate()

        # Calculate number of tests run, errors and failures based on output.
        res = re.search('Ran\s*(\d*)\s*test', err)
        nt = 1
        if res:
            nt = int(res.group(1))
        total += nt
        res = re.search('failures=(\d*)', err)
        nfail = 0
        if res:
            nfail = int(res.group(1))
        nerr = 0
        res = re.search('errors=(\d*)', err)
        if res:
            nerr = int(res.group(1))

        tot_err += nerr
        tot_fail += nfail

        # Print output catching any errors.
        if verbose > 0:
            nsuccess = nt - nerr - nfail
            res = '.'*nsuccess + 'F'*nfail + 'E'*nerr
            sys.stdout.write(res)
        if st != 0:
            errors.append([test, st, out, err, t2 - t1])

        if verbose > 1:
            print(out)
            print(err)
        sys.stdout.flush()
        total_time += t2 - t1

    print('\n' + '-'*70)
    print("Ran %d tests in %.4g seconds\n"%(total, total_time))

    errorcode = 0
    if len(errors) > 0:
        print('FAILED: there were %d failures and %d errors'\
                %(tot_fail, tot_err))
        for err in errors:
            test, st, out, err, t = err
            print('File:', test)
            print(out)
            print(err)
        errorcode = 1
    else:
        print('OK')
    return errorcode

def m2_tests(verbose=1):
    """Run all the TVTK and mayavi tests.
    """
    if verbose > 0:
        print("-"*70)
        print("Running TVTK tests.")
    tests = find_tests(['tvtk'])
    err = run(tests, verbose)
    if verbose > 0:
        print("-"*70)
        print("Running Mayavi tests.")
    tests = find_tests(['mayavi'])
    err += run(tests, verbose)
    return err

def main():
    usage = """%prog [options] directories/files/modules

    This program runs any tests in the given files and directories and
    also scans a module's directory for directories called 'tests' or
    'test' which contain 'test_*.py' files and runs them each in a
    separate subprocess.

    """
    parser = optparse.OptionParser(usage)
    parser.add_option("-v", "--verbose",
                      action="store_true",
                      default=False,
                      dest="verbose",
                      help="run tests in verbose mode")
    parser.add_option("-q", "--quiet",
                      action="store_true",
                      default=False,
                      dest="quiet",
                      help="run tests in quiet mode")
    parser.add_option("-n", "--nose",
                      action="store_true",
                      default=False,
                      dest="nose",
                      help="run tests using nose if it is available")

    options, args = parser.parse_args()
    verbose = 1
    global USE_NOSE
    if options.nose:
        USE_NOSE = True
    if options.verbose:
        verbose = 2
    if options.quiet:
        verbose = 0

    status = 0
    if len(args) == 0:
        status = m2_tests(verbose)
    else:
        tests = find_tests(args)
        if len(tests) > 0:
            status = run(tests, verbose=verbose)
    sys.exit(status)

if __name__ == "__main__":
    main()



from collections import defaultdict


class Bridge(object):
    def add_widget(self, scene_id, widget):
        '''Add widget corresponding to a given scene id.
        '''
        pass

    def remove_widget(self, scene_id, widget):
        '''Remove a widget corresponding to a given scene_id.
        '''
        pass

    def get_scenes(self):
        '''Returns a list of the available scenes.
        '''
        pass

    def handle_event(self, event_data):
        '''Handle an event sent by the server.
        '''
        pass

    def run(self):
        '''Run any code for the bridge to do its job.
        '''
        pass


class LocalBridge(Bridge):
    def __init__(self, scene_manager):
        self.scene_manager = scene_manager
        # In the local case, we are the client of
        # the scene manager and manage those events.
        self.scene_manager.add_client(self)
        self.widgets = defaultdict(list)

    def add_widget(self, scene_id, widget):
        '''Add widget corresponding to a given scene id.
        '''
        self.widgets[scene_id].append(widget)

    def remove_widget(self, scene_id, widget):
        '''Remove a widget corresponding to a given scene_id.
        '''
        w = self.widgets[scene_id]
        w.remove(widget)
        if len(w) == 0:
            del self.widgets[scene_id]

    def get_scenes(self):
        '''Returns a list of the available scenes.
        '''
        return self.scene_manager.scenes

    def handle_event(self, event_data):
        s_id, obj_name, event, data = event_data
        for w in self.widgets[s_id]:
            w.handle_vtk_event(obj_name, event, data)



"""Various preference helpers for the mayavi preferences.
"""
# Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
# Copyright (c) 2008-2015,  Enthought, Inc.
# License: BSD Style.

# Enthought library imports
from traits.api import (Bool, Enum, Tuple, Range, List,
        Str, Instance, HasTraits)
from traitsui.api import (View, Group, Item, RGBColorEditor,
        InstanceEditor)
from apptools.preferences.api import PreferencesHelper

################################################################################
# `RootPreferencesHelper` class
################################################################################
class RootPreferencesHelper(PreferencesHelper):

    # The preferences path for which we use.
    preferences_path = 'mayavi'

    ######################################################################
    # Our preferences.

    # Specifies if the nodes on the tree may be deleted without a
    # confirmation or not.  If True the user will be prompted before
    # the object is deleted.  If it is False then the user will not be
    # prompted.
    confirm_delete = Bool(desc='if the user is prompted before'
                          ' a node on the MayaVi tree is deleted')

    # Specifies if the splash screen is shown when mayavi starts.
    show_splash_screen = Bool(desc='if the splash screen is shown at'
                              ' startup')

    # Specifies if the adder nodes are shown on the mayavi tree view.
    show_helper_nodes = Bool(desc='if the helper (adder) nodes are shown'
                             ' on the tree view')

    # Specifies if the adder nodes are shown on the mayavi tree view.
    open_help_in_light_browser = Bool(
                    desc='if the help pages are opened in a chromeless'
                         ' browser window (only works with Firefox)')

    # Contrib directories to load on startup.
    contrib_packages = List(Str, desc='contrib packages to load on startup')


    # Whether or not to use IPython for the Shell.
    use_ipython = Bool(desc='use IPython for the embedded shell '
                            '(if available)')

    ########################################
    # Private traits.
    _contrib_finder = Instance(HasTraits)

    ######################################################################
    # Traits UI view.

    traits_view = View(
                    Group(
                        Item(name='confirm_delete'),
                        Item(name='show_splash_screen'),
                        Item(name='show_helper_nodes'),
                        Item(name='open_help_in_light_browser'),
                        Item('_contrib_finder',
                             show_label=False,
                             editor=InstanceEditor(label='Find contributions'),
                             )
                         ),
                    resizable=True
                    )

    ######################################################################
    # Non-public interface.
    ######################################################################
    def __contrib_finder_default(self):
        from .contrib_finder import ContribFinder
        return ContribFinder()

################################################################################
# `MlabPreferencesHelper` class
################################################################################
class MlabPreferencesHelper(PreferencesHelper):

    # The preferences path for which we use.
    preferences_path = 'mayavi.mlab'

    ######################################################################
    # Our preferences.

    # The mlab backend to use.
    backend = Enum('auto', 'envisage', 'simple', 'test',
                   desc='the mlab backend to use')

    # The background color of the renderer.
    background_color = Tuple(Range(0., 1.), Range(0., 1.), Range(0., 1.),
                             editor=RGBColorEditor,
                             desc='the background color of the scene')

    # The foreground color of the renderer.
    foreground_color = Tuple(Range(0., 1.), Range(0., 1.), Range(0., 1.),
                             editor=RGBColorEditor,
                             desc='the foreground color of the scene')

    # Offscreen rendering.
    offscreen = Bool(desc='if mlab should use offscreen rendering'
                          ' (no window will show up in this case)')

    ######################################################################
    # Traits UI view.

    traits_view = View(Group(
                             Item('backend'),
                             Item('background_color'),
                             Item('foreground_color'),
                             Item('offscreen')
                             ),
                       resizable=True
                      )



"""This module generates tvtk (Traited VTK) classes from the VTK-Python API.

"""
# Author: Prabhu Ramachandran
# Copyright (c) 2004-2015, Enthought, Inc.
# License: BSD Style.

from __future__ import print_function

import vtk_module as vtk
import os
import os.path
import zipfile
import tempfile
import shutil
import glob
import logging
from optparse import OptionParser
import sys

# Local imports -- these should be relative imports since these are
# imported before the package is installed.
try:
    from .common import get_tvtk_name, camel2enthought
    from .wrapper_gen import WrapperGenerator
    from .special_gen import HelperGenerator
except SystemError:
    from common import get_tvtk_name, camel2enthought
    from wrapper_gen import WrapperGenerator
    from special_gen import HelperGenerator


logger = logging.getLogger(__name__)

######################################################################
# `TVTKGenerator`
######################################################################


class TVTKGenerator:
    """Generates all the TVTK code."""

    def __init__(self, out_dir=''):
        """Initializes the instance.

        Parameters
        ----------

        - out_dir -  `string`

          The output directory to generate code in.  The directory is
          created if it does not exist.  A directory called
          `tvtk_classes` is created inside this directory and all the
          code is written here.  Any existing code there is blindly
          overwritten.  If no out_dir is specified, a temporary one is
          created using `tempfile.mkdtemp`.

        """
        if not out_dir:
            out_dir = tempfile.mkdtemp()
        self.out_dir = os.path.join(out_dir, 'tvtk_classes')

        if not os.path.exists(self.out_dir):
            os.makedirs(self.out_dir)
        self.zip_name = 'tvtk_classes.zip'

        self.wrap_gen = WrapperGenerator()
        self.helper_gen = HelperGenerator()

    #################################################################
    # `TVTKGenerator` interface.
    #################################################################

    def generate_code(self):
        """Generate all the wrapper code in `self.out_dir`.
        """
        out_dir = self.out_dir
        helper_gen = self.helper_gen
        wrap_gen = self.wrap_gen
        # Create an __init__.py file
        with open(os.path.join(out_dir, '__init__.py'), 'w'):
            pass

        # Crete a vtk_version.py file that contains VTK build
        # information.
        v = vtk.vtkVersion()
        vtk_version = v.GetVTKVersion()[:3]
        vtk_src_version = v.GetVTKSourceVersion()
        code = "vtk_build_version = \'%s\'\n"%(vtk_version)
        code += "vtk_build_src_version = \'%s\'\n"%(vtk_src_version)

        with open(os.path.join(out_dir, 'vtk_version.py'), 'w') as f:
            f.write(code)

        # Write the helper code header.
        with open(os.path.join(out_dir, 'tvtk_helper.py'), 'w') as helper_file:
            helper_gen.write_prelims(helper_file)

            # Write the wrapper files.
            tree = wrap_gen.get_tree().tree

            classes = []
            # This is another class we should not wrap and exists
            # in version 8.1.0.
            ignore = ['vtkOpenGLGL2PSHelperImpl']
            include = ['VTKPythonAlgorithmBase']
            for node in wrap_gen.get_tree():
                name = node.name
                if name in ignore:
                    continue
                if (name not in include and not name.startswith('vtk')) or \
                    name.startswith('vtkQt'):
                    continue
                if not hasattr(vtk, name) or not hasattr(getattr(vtk, name), 'IsA'):  # noqa
                    # We need to wrap VTK classes that are derived
                    # from vtkObjectBase, the others are
                    # straightforward VTK classes that can be used as
                    # such.  All of these have an 'IsA' method so we
                    # check for that.  Only the vtkObjectBase
                    # subclasses support observers etc. and hence only
                    # those make sense to wrap into TVTK.
                    continue
                classes.append(name)

            for nodes in tree:
                for node in nodes:
                    if node.name in classes:
                        tvtk_name = get_tvtk_name(node.name)
                        logger.debug(
                            'Wrapping %s as %s' % (node.name, tvtk_name))
                        self._write_wrapper_class(node, tvtk_name)
                        helper_gen.add_class(tvtk_name, helper_file)

    def write_wrapper_classes(self, names):
        """Given VTK class names in the list `names`, write out the
        wrapper classes to a suitable file.  This is a convenience
        method so one can generate a just a few of the wrapper classes
        if desired.  This is useful when debugging.  Please note that
        the method also generates code for all the ancestors of the
        specified classes.

        """
        # Wrappers for the accesors are generated in order to get the
        # _updateable_traits_ information correctly.
        nodes = []
        for name in names:
            node = self.wrap_gen.get_tree().get_node(name)
            if node is None:
                print('ERROR: Cannot find class: %s' % name)
            nodes.append(node)

        # Get ancestors.
        for node in nodes[:]:
            anc = node.get_ancestors()
            for i in anc:
                if i not in nodes:
                    nodes.insert(0, i)
        # Sort them as per their level.
        nodes.sort(key=lambda x: x.level)

        # Write code.
        for node in nodes:
            tvtk_name = get_tvtk_name(node.name)
            self._write_wrapper_class(node, tvtk_name)

    def build_zip(self, include_src=False):
        """Build the zip file (with name `self.zip_name`) in the
        current directory.

        Parameters
        ----------
        include_src : `bool` (default: False)
          If True, also includes all the ``*.py`` files in the ZIP file.
          By default only the ``*.pyc`` files are included.

        """
        cwd = os.getcwd()
        d = os.path.dirname(self.out_dir)
        os.chdir(d)
        z = zipfile.PyZipFile(self.zip_name, 'w',
                              zipfile.ZIP_DEFLATED)
        if include_src:
            l = glob.glob(os.path.join('tvtk_classes', '*.py'))
            for x in l:
                fname = os.path.basename(x)
                z.write(x, 'tvtk_classes/%s' % fname)
        z.writepy('tvtk_classes')
        z.close()
        if os.path.exists(cwd + "/" + self.zip_name):
            os.unlink(cwd + "/" + self.zip_name)
        shutil.move(self.zip_name, cwd)
        os.chdir(cwd)

    def clean(self):
        """Delete the temporary directory where the code has been
        generated.
        """
        tmp_dir = os.path.dirname(self.out_dir)
        d = os.listdir(tmp_dir)
        ok = 0
        if len(d) == 1 and d[0] == 'tvtk_classes':
            ok = 1
        if ok:
            shutil.rmtree(tmp_dir)
        else:
            print("Not removing directory:", tmp_dir)
            print("It does not contain a tvtk_classes directory!")

    #################################################################
    # Non-public interface.
    #################################################################
    def _write_wrapper_class(self, node, tvtk_name):
        """Write the wrapper code to a file."""
        # The only reason this method is separate is to generate code
        # for an individual class when debugging.
        fname = camel2enthought(tvtk_name) + '.py'
        if sys.version_info[0] > 2:
            out = open(os.path.join(self.out_dir, fname), 'w', encoding='utf-8')
        else:
            out = open(os.path.join(self.out_dir, fname), 'w')
        self.wrap_gen.generate_code(node, out)
        out.close()


######################################################################
# Utility functions.
######################################################################

def main():
    usage = """usage: %prog [options] [vtk_classes]

The options are described below.  An optional list of VTK classes for
which code is to be generated may be specified.  If none are specified
code will be generated for all the VTK classes.
    """
    parser = OptionParser(usage)
    parser.add_option(
        "-o", "--output-dir", action="store",
        type="string", dest="out_dir", default='',
        help="Output directory in which to generate code.")
    parser.add_option(
        "-n", "--no-clean", action="store_false",
        dest="clean", default=True,
        help="Do not clean the temporary directory.")
    parser.add_option(
        "-z", "--no-zipfile", action="store_false",
        dest="zip", default=True,
        help="Do not create a ZIP file.")
    parser.add_option(
        "-s", "--source", action="store_true",
        dest="src", default=False,
        help="Include source files (*.py) in "
             "addition to *.pyc files in the ZIP file.")
    parser.add_option(
        "-v", "--verbose", action="store_true",
        dest="verbose", default=False,
        help="Verbose output for debugging")

    (options, args) = parser.parse_args()

    if options.verbose:
        # Log to console
        logger = logging.getLogger()
        logger.setLevel(logging.DEBUG)
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        logger.addHandler(ch)

    # Now do stuff.
    gen = TVTKGenerator(options.out_dir)

    if len(args) == 0:
        gen.generate_code()
    else:
        gen.write_wrapper_classes(args)

    if options.zip:
        gen.build_zip(options.src)

    if options.clean:
        gen.clean()


if __name__ == '__main__':
    main()



"""
Test for the dataset_manager.py module.
"""
# Author: Prabhu Ramachandran <prabhu@aero.iitb.ac.in>
# Copyright (c) 2008-2015, Enthought, Inc.
# License: BSD Style.

import unittest
import numpy as N

from tvtk.api import tvtk
from mayavi.core.dataset_manager import DatasetManager


def make_data():
    points = N.array([[0,0,0], [1,0,0], [0,1,0], [0,0,1], # tets
                    [1,0,0], [2,0,0], [1,1,0], [1,0,1],
                    [2,0,0], [3,0,0], [2,1,0], [2,0,1],
                    ], 'f')
    tets = N.array([[0, 1, 2, 3], [4, 5, 6, 7], [8, 9, 10, 11]])
    tet_type = tvtk.Tetra().cell_type
    ug = tvtk.UnstructuredGrid(points=points)
    ug.set_cells(tet_type, tets)
    # Setup the point attributes.
    temp = N.random.random(12)
    v = N.random.randn(12, 3)
    ten = N.random.randn(12, 9)
    a = tvtk.FloatArray(name='p')
    a.from_array(N.random.randn(12))
    ug.point_data.add_array(a)
    ug.point_data.scalars = temp
    ug.point_data.scalars.name = 't'
    ug.point_data.vectors = v
    ug.point_data.vectors.name = 'v'
    ug.point_data.tensors = ten
    ug.point_data.tensors.name = 'ten'
    # Setup the cell attributes.
    temp = N.random.random(3)
    v = N.random.randn(3, 3)
    ten = N.random.randn(3, 9)
    ug.cell_data.scalars = temp
    ug.cell_data.scalars.name = 't'
    ug.cell_data.vectors = v
    ug.cell_data.vectors.name = 'v'
    ug.cell_data.tensors = ten
    ug.cell_data.tensors.name = 'ten'
    return ug


class TestDataSetManager(unittest.TestCase):
    def setUp(self):
        self.data = make_data()
        self.dm = DatasetManager(dataset=self.data)
        return

    def tearDown(self):
        return

    def test_point_arrays(self):
        "Are the point data arrays of the manager setup right?"
        dm = self.dm
        sc = sorted(dm.point_scalars.keys())
        self.assertEqual(sc, ['p', 't'])
        vec = list(dm.point_vectors.keys())
        self.assertEqual(vec, ['v'])
        ten = list(dm.point_tensors.keys())
        self.assertEqual(ten, ['ten'])

    def test_point_array_view(self):
        "Are the manager's point arrays views of the VTK data?"
        # Check that the array we have is really a view of the VTK data.
        dm = self.dm
        data = self.data
        t = dm.point_scalars['t']
        t[0] += 1.0
        self.assertEqual(t[0], data.point_data.scalars[0])

        v = dm.point_vectors['v']
        v[0][0] += 1.0
        self.assertEqual(v[0][0], data.point_data.vectors[0][0])

        ten = dm.point_tensors['ten']
        ten[0][0] += 1.0
        self.assertEqual(ten[0][0], data.point_data.tensors[0][0])

    def test_cell_arrays(self):
        "Are the cell arrays of the manager setup right?"
        dm = self.dm
        sc = list(dm.cell_scalars.keys())
        self.assertEqual(sc, ['t'])
        vec = list(dm.cell_vectors.keys())
        self.assertEqual(vec, ['v'])
        ten = list(dm.cell_tensors.keys())
        self.assertEqual(ten, ['ten'])

    def test_cell_array_view(self):
        "Are the manager's cell arrays views of the VTK data?"
        # Check that the array we have is really a view of the VTK data.
        dm = self.dm
        data = self.data
        t = dm.cell_scalars['t']
        t[0] += 1.0
        self.assertEqual(t[0], data.cell_data.scalars[0])

        v = dm.cell_vectors['v']
        v[0][0] += 1.0
        self.assertEqual(v[0][0], data.cell_data.vectors[0][0])

        ten = dm.cell_tensors['ten']
        ten[0][0] += 1.0
        self.assertEqual(ten[0][0], data.cell_data.tensors[0][0])

    def test_remove_array(self):
        "Does the remove_array method work correctly."
        dm = self.dm
        data = self.data
        dm.remove_array('t', 'point')
        self.assertEqual(len(dm.point_scalars), 1)
        self.assertEqual(list(dm.point_scalars.keys()), ['p'])
        dm.remove_array('ten', 'point')
        self.assertEqual(len(dm.point_tensors), 0)
        # Make sure the default category is point.
        dm.remove_array('v')
        self.assertEqual(len(dm.point_vectors), 0)

        # Cell arrays.
        dm.remove_array('t', 'cell')
        self.assertEqual(len(dm.cell_scalars), 0)
        dm.remove_array('ten', 'cell')
        self.assertEqual(len(dm.cell_tensors), 0)
        dm.remove_array('v', 'cell')
        self.assertEqual(len(dm.cell_vectors), 0)

    def test_rename_array(self):
        "Does array renaming work."
        dm = self.dm
        data = self.data

        dm.rename_array('ten', 'ten1', 'point')
        self.assertEqual(list(dm.point_tensors.keys()), ['ten1'])
        pd = data.point_data
        arrs = [pd.get_array_name(x) for x in
                range(pd.number_of_arrays)]
        arrs.sort()
        self.assertEqual(arrs, ['p', 't', 'ten1', 'v'])

        dm.rename_array('t', 'temp', 'cell')
        self.assertEqual(list(dm.cell_scalars.keys()), ['temp'])
        cd = data.cell_data
        arrs = [cd.get_array_name(x) for x in
                range(cd.number_of_arrays)]
        arrs.sort()
        self.assertEqual(arrs, ['temp', 'ten', 'v'])

    def test_add_array(self):
        "Does the add_array method work."
        dm = self.dm
        data = self.data
        pd = data.point_data
        cd = data.cell_data

        # Point data.
        s = N.random.randn(12)
        v = N.random.randn(12,3)
        t = N.random.randn(12, 9)
        dm.add_array(s, 'scalar')
        sc = sorted(self.dm.point_scalars.keys())
        self.assertEqual(sc, ['p', 'scalar', 't'])
        x = pd.get_array('scalar')
        self.assertNotEqual(x, None)
        dm.add_array(v, 'vector')
        vc = sorted(self.dm.point_vectors.keys())
        self.assertEqual(vc, ['v', 'vector'])
        x = pd.get_array('vector')
        self.assertNotEqual(x, None)
        dm.add_array(t, 'tensor')
        vc = sorted(self.dm.point_tensors.keys())
        self.assertEqual(vc, ['ten', 'tensor'])
        x = pd.get_array('tensor')
        self.assertNotEqual(x, None)

        # cell data.
        s = N.random.randn(3)
        v = N.random.randn(3, 3)
        t = N.random.randn(3, 9)
        dm.add_array(s, 'scalar', 'cell')
        sc = sorted(self.dm.cell_scalars.keys())
        self.assertEqual(sc, ['scalar', 't'])
        x = cd.get_array('scalar')
        self.assertNotEqual(x, None)
        dm.add_array(v, 'vector', 'cell')
        vc = sorted(self.dm.cell_vectors.keys())
        self.assertEqual(vc, ['v', 'vector'])
        x = cd.get_array('vector')
        self.assertNotEqual(x, None)
        dm.add_array(t, 'tensor', 'cell')
        vc = sorted(self.dm.cell_tensors.keys())
        self.assertEqual(vc, ['ten', 'tensor'])
        x = cd.get_array('tensor')
        self.assertNotEqual(x, None)

    def test_activate(self):
        "Does activating a particular array work."
        dm = self.dm
        data = self.dm.output

        dm.activate('p')
        self.assertEqual(data.point_data.scalars.name, 'p')
        dm.activate('t')
        self.assertEqual(data.point_data.scalars.name, 't')

        s = N.random.randn(3)
        dm.add_array(s, 'foo', 'cell')
        dm.activate('foo', 'cell')
        self.assertEqual(data.cell_data.scalars.name, 'foo')
        dm.activate('t', 'cell')
        self.assertEqual(data.cell_data.scalars.name, 't')


if __name__ == '__main__':
    unittest.main()



# Author: Suyog Dutt Jain <suyog.jain@aero.iitb.ac.in>
#         Prabhu Ramachandran <prabhu_r@users.sf.net>
# Copyright (c) 2008-2015,  Enthought, Inc.
# License: BSD Style.

# Standard library imports.
from os.path import abspath
from io import BytesIO
import copy
import numpy
import unittest

# Enthought library imports
from mayavi.core.null_engine import NullEngine
from mayavi.sources.array_source import ArraySource
from mayavi.modules.outline import Outline
from mayavi.modules.streamline import Streamline

class TestStreamline(unittest.TestCase):


    def make_data(self):
        """Trivial data -- creates an elementatry scalar field and a
        constant vector field along the 'x' axis."""
        s = numpy.arange(0.0, 10.0, 0.01)
        s = numpy.reshape(s, (10,10,10))
        s = numpy.transpose(s)

        v = numpy.zeros(3000, 'd')
        v[1::3] = 1.0
        v = numpy.reshape(v, (10,10,10,3))
        return s, v

    def setUp(self):
        """Initial setting up of test fixture, automatically called by TestCase before any other test method is invoked"""
        e = NullEngine()
        # Uncomment to see visualization for debugging etc.
        #e = Engine()
        e.start()
        s=e.new_scene()
        self.e=e
        self.s=s

        ############################################################
        # Create a new scene and set up the visualization.

        d = ArraySource()
        sc, vec = self.make_data()
        d.origin = (-5, -5, -5)
        d.scalar_data = sc
        d.vector_data = vec

        e.add_source(d)

        # Create an outline for the data.
        o = Outline()
        e.add_module(o)

        # View the data.
        st = Streamline()
        e.add_module(st)
        widget = st.seed.widget
        widget.trait_set(radius=1.0, center=(-4.0, -4.0, -4.0),
                   theta_resolution=4, phi_resolution=4)

        st = Streamline(streamline_type='ribbon')
        seed = st.seed
        seed.widget = seed.widget_list[1]
        e.add_module(st)
        seed.widget.trait_set(point1=(-5.0, -4.5, -4.0), point2=(-5.0, -4.5, 4.0))
        st.ribbon_filter.width = 0.25

        st = Streamline(streamline_type='tube')
        seed = st.seed
        seed.widget = seed.widget_list[2]
        e.add_module(st)
        seed.widget.trait_set(center=(-5.0, 1.5, -2.5))
        st.tube_filter.radius = 0.15

        st = Streamline(streamline_type='tube')
        seed = st.seed
        seed.widget = seed.widget_list[3]
        e.add_module(st)
        seed.widget.position=(-5.0, 3.75, 3.75)
        st.tube_filter.radius = 0.2
        self.st = st
        self.scene = e.current_scene
        return

    def tearDown(self):
        """For necessary clean up, automatically called by TestCase after the test methods have been invoked"""
        self.e.stop()
        return

    def check(self):
        """Do the actual testing."""

        s=self.scene
        src = s.children[0]

        st = src.children[0].children[2]
        self.assertEqual(st.streamline_type,'ribbon')
        self.assertEqual(st.ribbon_filter.width,0.25)
        self.assertEqual(st.seed.widget,st.seed.widget_list[1])
        self.assertEqual(numpy.allclose(st.seed.widget.point1,
                            (-5.0, -4.5, -4.0)),True)
        self.assertEqual(numpy.allclose(st.seed.widget.point2,
                            (-5.0, -4.5, 4.0)),True)

        st = src.children[0].children[3]
        self.assertEqual(st.streamline_type,'tube')
        self.assertEqual(st.tube_filter.radius,0.15)
        self.assertEqual(st.seed.widget,st.seed.widget_list[2])
        self.assertEqual(numpy.allclose(st.seed.widget.center,
                            (-5.0, 1.5, -2.5)),True)

        st = src.children[0].children[4]
        self.assertEqual(st.streamline_type,'tube')
        self.assertEqual(st.tube_filter.radius,0.2)
        self.assertEqual(st.seed.widget,st.seed.widget_list[3])
        self.assertEqual(numpy.allclose(st.seed.widget.position,
                            (-5.0, 3.75, 3.75)),True)


    def test_streamline(self):
        "Test if the test fixture works"
        self.check()

    def test_components_changed(self):
        """Test if the modules respond correctly when the components
           are changed."""
        st = self.st
        tf = st.tube_filter
        st.tube_filter = tf.__class__()
        st.tube_filter = tf
        st.ribbon_filter = st.ribbon_filter.__class__()
        seed = st.seed
        st.seed = seed.__class__()
        st.seed = seed
        st.actor = st.actor.__class__()
        tracer = st.stream_tracer
        st.stream_tracer = tracer.__class__()
        st.stream_tracer = tracer
        self.check()



    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""
        engine = self.e
        scene = self.scene
        # Save visualization.
        f = BytesIO()
        f.name = abspath('test.mv2') # We simulate a file.
        engine.save_visualization(f)
        f.seek(0) # So we can read this saved data.

        # Remove existing scene.

        engine.close_scene(scene)

        # Load visualization
        engine.load_visualization(f)
        self.scene = engine.current_scene

        self.check()

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        # Pop the source object.
        s =  self.scene
        sources = s.children
        s.children = []
        # Add it back to see if that works without error.
        s.children.extend(sources)

        self.check()

        # Now deepcopy the source and replace the existing one with
        # the copy.  This basically simulates cutting/copying the
        # object from the UI via the right-click menu on the tree
        # view, and pasting the copy back.
        sources1 = copy.deepcopy(sources)
        s.children[:] = sources1
        self.check()


if __name__ == '__main__':
    unittest.main()



"""This source manages a VTK dataset given to it.  When this source is
pickled or persisted, it saves the data given to it in the form of a
gzipped string.
"""
# Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
# Copyright (c) 2005-2015, Enthought, Inc.
# License: BSD Style.

import sys
import os
import tempfile

import numpy as np

# Enthought library imports.
from traits.api import Instance, List, Str, Bool, Int
from traitsui.api import View, Group, Item
from apptools.persistence.state_pickler \
     import gzip_string, gunzip_string, set_state

# Local imports.
from tvtk.api import tvtk
from tvtk import messenger
from tvtk.array_handler import array2vtk
from tvtk.common import is_old_pipeline, configure_input_data
from mayavi.core.source import Source
from mayavi.core.common import handle_children_state
from mayavi.core.trait_defs import DEnum
from mayavi.core.pipeline_info import (PipelineInfo,
                                       get_tvtk_dataset_name)
from .utils import has_attributes
from .vtk_xml_file_reader import get_all_attributes


######################################################################
# Utility functions.
######################################################################
def write_dataset_to_string(data):
    """Given a dataset, convert the dataset to an ASCII string that can
    be stored for persistence.
    """
    w = tvtk.DataSetWriter(write_to_output_string=1)
    warn = w.global_warning_display
    configure_input_data(w, data)
    w.global_warning_display = 0
    w.update()
    if w.output_string_length == 0:
        # Some VTK versions (5.2) have a bug when writing structured
        # grid datasets and produce empty output.  We work around this
        # by writing to a file and then reading that output.
        w.write_to_output_string = 0
        fh, fname = tempfile.mkstemp('.vtk')
        os.close(fh); os.remove(fname)
        w.file_name = fname
        w.write()
        # Read the data and delete the file.
        sdata = open(fname).read()
        os.remove(fname)
    else:
        sdata = w.output_string
    w.global_warning_display = warn
    return sdata


######################################################################
# `VTKDataSource` class
######################################################################
class VTKDataSource(Source):

    """This source manages a VTK dataset given to it.  When this
    source is pickled or persisted, it saves the data given to it in
    the form of a gzipped string.

    Note that if the VTK dataset has changed internally and you need
    to notify the mayavi pipeline to flush the data just call the
    `modified` method of the VTK dataset and the mayavi pipeline will
    update automatically.
    """

    # The version of this class.  Used for persistence.
    __version__ = 0

    # The VTK dataset to manage.
    data = Instance(tvtk.DataSet, allow_none=False)

    # Information about what this object can produce.
    output_info = PipelineInfo(datasets=['any'],
                               attribute_types=['any'],
                               attributes=['any'])

    ########################################
    # Dynamic traits: These traits are dynamic and are updated on the
    # _update_data method.

    # The active point scalar name.
    point_scalars_name = DEnum(values_name='_point_scalars_list',
                               desc='scalar point data attribute to use')
    # The active point vector name.
    point_vectors_name = DEnum(values_name='_point_vectors_list',
                               desc='vectors point data attribute to use')
    # The active point tensor name.
    point_tensors_name = DEnum(values_name='_point_tensors_list',
                               desc='tensor point data attribute to use')

    # The active cell scalar name.
    cell_scalars_name = DEnum(values_name='_cell_scalars_list',
                               desc='scalar cell data attribute to use')
    # The active cell vector name.
    cell_vectors_name = DEnum(values_name='_cell_vectors_list',
                               desc='vectors cell data attribute to use')
    # The active cell tensor name.
    cell_tensors_name = DEnum(values_name='_cell_tensors_list',
                               desc='tensor cell data attribute to use')

    ########################################
    # Our view.

    view = View(Group(Item(name='point_scalars_name'),
                      Item(name='point_vectors_name'),
                      Item(name='point_tensors_name'),
                      Item(name='cell_scalars_name'),
                      Item(name='cell_vectors_name'),
                      Item(name='cell_tensors_name'),
                      Item(name='data'),
                      ))

    ########################################
    # Private traits.

    # These private traits store the list of available data
    # attributes.  The non-private traits use these lists internally.
    _point_scalars_list = List(Str)
    _point_vectors_list = List(Str)
    _point_tensors_list = List(Str)
    _cell_scalars_list = List(Str)
    _cell_vectors_list = List(Str)
    _cell_tensors_list = List(Str)

    # This filter allows us to change the attributes of the data
    # object and will ensure that the pipeline is properly taken care
    # of.  Directly setting the array in the VTK object will not do
    # this.
    _assign_attribute = Instance(tvtk.AssignAttribute, args=(),
                                 allow_none=False)

    # Toggles if this is the first time this object has been used.
    _first = Bool(True)

    # The ID of the observer for the data.
    _observer_id = Int(-1)

    ######################################################################
    # `object` interface
    ######################################################################
    def __get_pure_state__(self):
        d = super(VTKDataSource, self).__get_pure_state__()
        for name in ('_assign_attribute', '_first', '_observer'):
            d.pop(name, None)
        for name in ('point_scalars', 'point_vectors',
                     'point_tensors', 'cell_scalars',
                     'cell_vectors', 'cell_tensors'):
            d.pop('_' + name + '_list', None)
            d.pop('_' + name + '_name', None)
        data = self.data
        if data is not None:
            sdata = write_dataset_to_string(data)
            if sys.version_info[0] > 2:
                z = gzip_string(sdata.encode('ascii'))
            else:
                z = gzip_string(sdata)
            d['data'] = z
        return d

    def __set_pure_state__(self, state):
        z = state.data
        if z is not None:
            if sys.version_info[0] > 2:
                d = gunzip_string(z).decode('ascii')
            else:
                d = gunzip_string(z)
            r = tvtk.DataSetReader(read_from_input_string=1,
                                   input_string=d)
            warn = r.global_warning_display
            r.global_warning_display = 0
            r.update()
            r.global_warning_display = warn
            self.data = r.output
        # Now set the remaining state without touching the children.
        set_state(self, state, ignore=['children', 'data'])
        # Setup the children.
        handle_children_state(self.children, state.children)
        # Setup the children's state.
        set_state(self, state, first=['children'], ignore=['*'])

    ######################################################################
    # `Base` interface
    ######################################################################
    def start(self):
        """This is invoked when this object is added to the mayavi
        pipeline.
        """
        # Do nothing if we are already running.
        if self.running:
            return

        # Update the data just in case.
        self._update_data()

        # Call the parent method to do its thing.  This will typically
        # start all our children.
        super(VTKDataSource, self).start()

    def update(self):
        """Invoke this to flush data changes downstream.  This is
        typically used when you change the data object and want the
        mayavi pipeline to refresh.
        """
        # This tells the VTK pipeline that the data has changed.  This
        # will fire the data_changed event automatically.
        self.data.modified()
        if has_attributes(self.data):
            self._assign_attribute.update()

    def add_attribute(self, array, name, category='point'):
        """Add an attribute to the dataset to specified category ('point' or
        'cell').

        One may add a scalar, vector (3/4 components) or a tensor (9 components).

        Note that it is the user's responsibility to set the correct size of
        the arrays.

        Parameters
        ----------

        array: numpy array/list : array data to add.

        name: str: name of the array.

        category: 'point'/'cell': the category of the attribute data.

        """
        array = np.asarray(array)
        assert len(array.shape) <= 2, "Only 2D arrays can be added."
        data = getattr(self.data, '%s_data' % category)
        if len(array.shape) == 2:
            assert array.shape[1] in [1, 3, 4, 9], \
                    "Only Nxm arrays where (m in [1,3,4,9]) are supported"
            va = tvtk.to_tvtk(array2vtk(array))
            va.name = name
            data.add_array(va)
            mapping = {1: 'scalars', 3: 'vectors', 4: 'scalars',
                       9: 'tensors'}
            attribute = '_%s_%s_list' % (category, mapping[array.shape[1]])
        else:
            va = tvtk.to_tvtk(array2vtk(array))
            va.name = name
            data.add_array(va)
            attribute = '_%s_scalars_list' % category
        names = set(getattr(self, attribute) + [name])
        setattr(self, attribute, sorted(names))

    def remove_attribute(self, name, category='point'):
        """Remove an attribute by its name and optional category (point and
        cell).  Returns the removed array.
        """
        type = self._find_array_list(name, category)
        data = getattr(self.data, '%s_data' % category)
        data.remove_array(name)
        attr_list = getattr(self, '_%s_%s_list' % (category, type))
        return attr_list.remove(name)

    def rename_attribute(self, name1, name2, category='point'):
        """Rename a particular attribute from `name1` to `name2`.
        """
        type = self._find_array_list(name1, category)
        data = getattr(self.data, '%s_data' % category)
        arr = data.get_array(name1)
        arr.name = name2
        attribute = '_%s_%s_list' % (category, type)
        attr_list = getattr(self, attribute)
        attr_list.remove(name1)
        attr_list.append(name2)
        setattr(self, attribute, sorted(attr_list))

    ######################################################################
    # Non-public interface
    ######################################################################
    def _data_changed(self, old, new):
        if has_attributes(self.data):
            aa = self._assign_attribute
            self.configure_input_data(aa, new)
            self._update_data()
            aa.update()
            self.outputs = [aa]
        else:
            tp = tvtk.TrivialProducer()
            tp.set_output(self.data)
            self.outputs = [tp]
        self.data_changed = True

        self.output_info.datasets = [get_tvtk_dataset_name(self.outputs[0])]

        # Add an observer to the VTK dataset after removing the one
        # for the old dataset.  We use the messenger to avoid an
        # uncollectable reference cycle.  See the
        # tvtk.messenger module documentation for details.
        if old is not None:
            old.remove_observer(self._observer_id)
        self._observer_id = new.add_observer('ModifiedEvent',
                                             messenger.send)
        new_vtk = tvtk.to_vtk(new)
        messenger.connect(new_vtk, 'ModifiedEvent',
                          self._fire_data_changed)

        # Change our name so that our label on the tree is updated.
        self.name = self._get_name()

    def _fire_data_changed(self, *args):
        """Simply fire the `data_changed` event."""
        self.data_changed = True

    def _set_data_name(self, data_type, attr_type, value):
        if value is None:
            return

        dataset = self.data
        if len(value) == 0:
            # If the value is empty then we deactivate that attribute.
            d = getattr(dataset, attr_type + '_data')
            method = getattr(d, 'set_active_%s'%data_type)
            method(None)
            self.data_changed = True
            return

        aa = self._assign_attribute
        data = None
        if attr_type == 'point':
            data = dataset.point_data
        elif attr_type == 'cell':
            data = dataset.cell_data
        method = getattr(data, 'set_active_%s'%data_type)
        method(value)
        aa.assign(value, data_type.upper(), attr_type.upper() +'_DATA')
        if data_type == 'scalars' and dataset.is_a('vtkImageData'):
            # Set the scalar_type for image data, if not you can either
            # get garbage rendered or worse.
            s = getattr(dataset, attr_type + '_data').scalars
            r = s.range
            if is_old_pipeline():
                dataset.scalar_type = s.data_type
                aa.output.scalar_type = s.data_type
        aa.update()
        # Fire an event, so the changes propagate.
        self.data_changed = True

    def _point_scalars_name_changed(self, value):
        self._set_data_name('scalars', 'point', value)

    def _point_vectors_name_changed(self, value):
        self._set_data_name('vectors', 'point', value)

    def _point_tensors_name_changed(self, value):
        self._set_data_name('tensors', 'point', value)

    def _cell_scalars_name_changed(self, value):
        self._set_data_name('scalars', 'cell', value)

    def _cell_vectors_name_changed(self, value):
        self._set_data_name('vectors', 'cell', value)

    def _cell_tensors_name_changed(self, value):
        self._set_data_name('tensors', 'cell', value)

    def _update_data(self):
        if self.data is None:
            return
        pnt_attr, cell_attr = get_all_attributes(self.data)

        pd = self.data.point_data
        scalars = pd.scalars
        if self.data.is_a('vtkImageData') and scalars is not None:
            # For some reason getting the range of the scalars flushes
            # the data through to prevent some really strange errors
            # when using an ImagePlaneWidget.
            r = scalars.range
            if is_old_pipeline():
                self._assign_attribute.output.scalar_type = scalars.data_type
                self.data.scalar_type = scalars.data_type

        def _setup_data_traits(obj, attributes, d_type):
            """Given the object, the dict of the attributes from the
            `get_all_attributes` function and the data type
            (point/cell) data this will setup the object and the data.
            """
            attrs = ['scalars', 'vectors', 'tensors']
            aa = obj._assign_attribute
            data = getattr(obj.data, '%s_data'%d_type)
            for attr in attrs:
                values = sorted(attributes[attr])
                values.append('')
                setattr(obj, '_%s_%s_list'%(d_type, attr), values)
                if len(values) > 1:
                    default = getattr(obj, '%s_%s_name'%(d_type, attr))
                    if obj._first and len(default) == 0:
                        default = values[0]
                    getattr(data, 'set_active_%s'%attr)(default)
                    aa.assign(default, attr.upper(),
                              d_type.upper() +'_DATA')
                    aa.update()
                    kw = {'%s_%s_name'%(d_type, attr): default,
                          'trait_change_notify': False}
                    obj.trait_set(**kw)

        _setup_data_traits(self, pnt_attr, 'point')
        _setup_data_traits(self, cell_attr, 'cell')
        if self._first:
            self._first = False
        # Propagate the data changed event.
        self.data_changed = True

    def _get_name(self):
        """ Gets the name to display on the tree.
        """
        ret = "VTK Data (uninitialized)"
        if self.data is not None:
            typ = self.data.__class__.__name__
            ret = "VTK Data (%s)" % typ
        if '[Hidden]' in self.name:
            ret += ' [Hidden]'
        return ret

    def _find_array_list(self, name, category='point'):
        """Return information on which kind of attribute contains the
        specified named array in a particular category."""
        types = ['scalars', 'vectors', 'tensors']
        for type in types:
            attr = '_%s_%s_list' % (category, type)
            names = getattr(self, attr)
            if name in names:
                return type
        raise KeyError('No %s array named %s available in dataset'
                       % (category, name))



"""The base class for many filters.
"""
# Author: Prabhu Ramachandran <prabhu_r at users dot sf dot net>
# Copyright (c) 2006, Enthought, Inc.
# License: BSD Style.


# Enthought library imports.
from traits.api import Instance
from traitsui.api import View, Group, Item
from tvtk.api import tvtk

# Local imports
from mayavi.core.filter import Filter

######################################################################
# `FilterBase` class.
######################################################################
class FilterBase(Filter):
    """The base class for many of the filters.
    """

    # The version of this class.  Used for persistence.
    __version__ = 0

    # The actual TVTK filter that this class manages.
    filter = Instance(tvtk.Object, allow_none=False, record=True)

    # The view of these filters.

    view = View(Group(Item(name='filter', style='custom', resizable=True,
                      show_label=False), springy=True),
                scrollable=True,
                resizable=True
                )

    ######################################################################
    # `Filter` interface.
    ######################################################################
    def setup_pipeline(self):
        """Override this method so that it *creates* its tvtk
        pipeline.

        This method is invoked when the object is initialized via
        `__init__`.  Note that at the time this method is called, the
        tvtk data pipeline will *not* yet be setup.  So upstream data
        will not be available.  The idea is that you simply create the
        basic objects and setup those parts of the pipeline not
        dependent on upstream sources and filters.
        """
        f = self.filter
        if f is not None:
            # Just hook up the filter so the update_data method is
            # called when the traits change.
            f.on_trait_change(self.update_data)

    def update_pipeline(self):
        """Override this method so that it *updates* the tvtk pipeline
        when data upstream is known to have changed.

        This method is invoked (automatically) when the input fires a
        `pipeline_changed` event.
        """
        # Do nothing if there is no input.
        inputs = self.inputs
        fil = self.filter
        if len(inputs) == 0 or fil is None:
            return

        # By default we set the input to the first output of the first
        # input.
        self.configure_connection(fil, inputs[0])
        fil.update()
        self._set_outputs([fil])

    def update_data(self):
        """Override this method to do what is necessary when upstream
        data changes.

        This method is invoked (automatically) when any of the inputs
        sends a `data_changed` event.
        """
        # Do nothing if there is no input and we aren't running.
        if len(self.inputs) == 0 or not self.running:
            return

        self.filter.update()
        # Propagate the data_changed event.
        self.data_changed = True

    def _filter_changed(self, old, new):
        if old is not None:
            old.on_trait_change(self.update_data, remove=True)

        new.on_trait_change(self.update_data)

        if old is not None:
            self.update_pipeline()



"""A tvtk pipeline browser.

An abstract `TreeGenerator` class defines the interface of a tree
generator.  This class is responsible for generating the list of
children.  Often tvtk object's children are collections of various
objects, some sequences and some simple objects.  In order to provide
a unified interface to all children, all of these objects are wrapped
using the `CompositeIterable` which presents all children as a single
iterable.

`SimpleTreeGenerator` does not do extensive analysis of the passed
object in order to compute the children.  `FullTreeGenerator` however
uses the algorithm that MayaVi-1.x uses and therefore generates a
large number of objects.

The `PipelineBrowser` class presents the view of the pipeline as a
tree.  It allows one to specify the TreeGenerator instance.  The
`TreeEditor` from the traits package is used to represent the view.  A
`TVTKLeafNode` defines a node that has no children.  A
`TVTKBranchNode` is a node that has children.  The nodes basically
wrap up the tvtk object and present an interface suitable for the
TreeEditor.

TODO:

 * When a node is selected, the actor involved could be highlighted.

"""
# Author: Prabhu Ramachandran <prabhu@aero.iitb.ac.in>
# Copyright (c) 2005-2018, Enthought, Inc.
# License: BSD Style.

# Standard library imports.
import re

# Enthought library imports.
from traits.api import (HasTraits, Property, Any, Instance, Event,
                        Trait, List, Str, Dict, Python)
from traitsui.api import \
     TreeEditor, TreeNodeObject, ObjectTreeNode, View, Item, Group, VSplit
from traitsui.menu import Menu, Action

from tvtk.api import tvtk
from tvtk import messenger
from tvtk.tvtk_base import TVTKBase
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk.common import camel2enthought


######################################################################
# Utility functions.
######################################################################
def is_iterable(x):
    return hasattr(x, '__iter__')


def get_icon(object_name):
    """Given the name of the object, this function returns an
    appropriate icon image name.  If no icon is appropriate it returns
    the empty string."""

    # The mapping from names to icons.
    icon_map = {'actor': 'actor.png',
                'camera': 'camera.png',
                'coordinate': 'coordinate.png',
                'filter': 'filter.png',
                'lookuptable': 'lookuptable.png',
                'mapper': 'mapper.png',
                'polydata': 'polydata.png',
                'property': 'property.png',
                'reader': 'reader.png',
                'renderer': 'renderer.png',
                'rendererwindowinteractor': 'rendererwindowinteractor.png',
                'source': 'source.png',
                'texture': 'texture.png',
                'window': 'window.png',
                'writer': 'writer.png',
                }

    # Lower case the name.
    name = object_name.lower()

    for key in icon_map:
        if name.endswith(key):
            return icon_map[key]

    # No valid icon for this object.
    return ''


######################################################################
# `TreeGenerator` class.
######################################################################
class TreeGenerator(HasTraits):
    """Encapsulates the methods that generate the tree via the
    `get_children` method."""

    def has_children(self, obj):
        """Returns True if object `obj` has children."""
        raise NotImplementedError

    def get_children(self, obj):
        """Returns a dictionary containing the children of the object
        `obj`."""
        raise NotImplementedError

    def get_node(self, obj):
        """Get a node object representing this object."""
        raise NotImplementedError

    def get_nodes(self, menu):
        """Returns a list of nodes for the tree editor.  The menu
        entries to use for the nodes are passed as `menu`."""
        raise NotImplementedError


######################################################################
# `SimpleTreeGenerator` class.
######################################################################
class SimpleTreeGenerator(TreeGenerator):
    """This particular class generates a simple pipeline
    representation.  Not every possible object is obtained."""

    def has_children(self, obj):
        """Returns true of the object has children, false if not.  This is
        very specific to tvtk objects."""
        if isinstance(obj, (tvtk.RenderWindow, tvtk.Renderer,
                            tvtk.Collection)):
            return True

        for attribute in ['source', 'get_input', 'input', 'mapper', 'property',
                          'texture', 'text_property', 'volume_property',
                          'lookup_table', 'producer_port', 'producer',
                          'get_input_algorithm']:
            if hasattr(obj, attribute):
                return True

        return False

    def get_children(self, obj):
        """Returns the child objects of a particular tvtk object in a
        dictionary, the keys are the trait names.  This is used to
        generate the tree in the browser."""
        kids = {}
        def _add_kid(key, x):
            if x is None:
                kids[key] = None
            else:
                if type(x) in (type([]), type(())):
                    x1 = [i for i in x if isinstance(i, TVTKBase)]
                    if x1:
                        kids[key] = x1
                elif isinstance(x, TVTKBase):
                    kids[key] = x

        if isinstance(obj, tvtk.RenderWindow):
            return {'renderers':obj.renderers}
        elif isinstance(obj, tvtk.Renderer):
            if hasattr(obj, 'view_props'):
                return {'view_props':obj.view_props,
                        'active_camera':obj.active_camera}
            else:
                return {'props':obj.props,
                        'active_camera':obj.active_camera}

        #if isinstance(obj, tvtk.Collection):
        #    _add_kid(obj)

        # Misc. properties.
        for attribute in ['mapper', 'property', 'texture',
                          'text_property', 'volume_property',
                          'lookup_table', 'producer']:
            if hasattr(obj, attribute):
                _add_kid(attribute, getattr(obj, attribute))

        # Check for sources and inputs.
        if hasattr(obj, 'number_of_sources'):
            srcs = [obj.get_source(i)
                    for i in range(obj.number_of_sources)]
            _add_kid('source', srcs)
        elif hasattr(obj, 'source'):
            _add_kid('source', obj.source)

        if hasattr(obj, 'get_input_algorithm'):
            inputs = []
            if hasattr(obj, 'number_of_input_ports'):
                inputs = [obj.get_input_algorithm(i, j)
                          for i in range(obj.number_of_input_ports)
                          for j in range(
                                  obj.get_number_of_input_connections(i))]
            elif hasattr(obj, 'get_input'):
                inputs = [obj.get_input(i)
                          for i in range(obj.number_of_inputs)]
            _add_kid('input', inputs)
        elif hasattr(obj, 'input'):
            _add_kid('input', obj.input)

        if hasattr(obj, 'producer_port'):
            _add_kid('producer_port', obj.producer_port)

        return kids

    def get_node(self, obj):
        """Get a node object representing the object passed."""
        if self.has_children(obj):
            return TVTKBranchNode(object=obj, tree_generator=self)
        else:
            return TVTKLeafNode(object=obj)

    def get_nodes(self, menu):
        """Returns a list of nodes for the tree editor.  The menu
        entries to use are given as `menu`"""
        nodes = [ObjectTreeNode(node_for=[TVTKBranchNode],
                                view=View(Group(Item('object', style='custom'),
                                                show_labels=False)),
                                auto_open=False,
                                children='children', label='name', menu=menu,
                                rename=False, delete=False, copy=False,
                                insert=False),
                 ObjectTreeNode(node_for=[TVTKLeafNode],
                                view=View(Group(Item('object', style='custom'),
                                                show_labels=False)),
                                auto_open=False,
                                label='name', menu=menu, rename=False,
                                delete=False, copy=False, insert=False),
                 ObjectTreeNode(node_for=[TVTKCollectionNode],
                                auto_open=True, children='children',
                                label='name', menu=menu, rename=False,
                                delete=False, copy=False, insert=False),
                 ]
        return nodes


######################################################################
# `FullTreeGenerator` class.
######################################################################
class FullTreeGenerator(SimpleTreeGenerator):
    """This particular class picks up a lot more children in the
    pipeline and is similar to the code used in MayaVi-1.x's pipeline
    browser."""

    def __init__(self, **traits):
        super(FullTreeGenerator, self).__init__(**traits)
        self.last_transform = 0

    def get_children(self, obj):
        """Returns the child objects of a particular tvtk object in a
        dictionary, the keys are the trait names.  This is used to
        generate the tree in the browser."""

        vtk_obj = tvtk.to_vtk(obj)
        methods = self._get_methods(vtk_obj)

        kids = {}
        def _add_kid(key, x):
            if x is None:
                kids[key] = None
            else:
                if type(x) in (type([]), type(())):
                    x1 = [i for i in x if isinstance(i, TVTKBase)]
                    if x1:
                        kids[key] = x1
                elif isinstance(x, TVTKBase):
                    if hasattr(x, '__iter__'):
                        # Don't add iterable objects that contain non
                        # acceptable nodes
                        if len(list(x)) and isinstance(list(x)[0], TVTKBase):
                            kids[key] = x
                    else:
                        kids[key] = x

        for method in methods:
            attr = camel2enthought(method[0])
            if hasattr(obj, attr):
                _add_kid(attr, getattr(obj, attr))

        # Check for sources and inputs.
        if hasattr(obj, 'number_of_sources'):
            srcs = [obj.get_source(i)
                    for i in range(obj.number_of_sources)]
            _add_kid('source', srcs)
        elif hasattr(obj, 'source'):
            _add_kid('source', obj.source)

        if hasattr(obj, 'get_input_algorithm'):
            inputs = []
            if hasattr(obj, 'number_of_input_ports'):
                inputs = [obj.get_input_algorithm(i, j)
                          for i in range(obj.number_of_input_ports)
                          for j in range(
                                  obj.get_number_of_input_connections(i))]
            _add_kid('input', inputs)
        elif hasattr(obj, 'get_input'):
            inputs = [obj.get_input(i)
                      for i in range(obj.number_of_inputs)]
            _add_kid('input', inputs)
        elif hasattr(obj, 'input'):
            _add_kid('input', obj.input)

        if hasattr(obj, 'producer_port'):
            _add_kid('producer_port', obj.producer_port)

        return kids

    def has_children(self, obj):
        """Returns true of the object has children, false if not.  This is
        very specific to tvtk objects."""
        result = super(FullTreeGenerator, self).has_children(obj)
        # FIXME: This is inefficient.  We probably should cache the
        # get_children call.
        if not result and self.get_children(obj):
            return True
        return result

    ###########################################################################
    # Non-public interface.
    ###########################################################################
    def _get_methods(self, vtk_obj):
        """Obtain the various methods from the passed object."""

        def _remove_method(name, methods, method_names):
            """Removes methods if they have a particular name."""
            try:
                idx = method_names.index(name)
            except ValueError:
                pass
            else:
                del methods[idx], method_names[idx]
            return methods, method_names

        # The following code basically gets the 'str' representation
        # of the VTK object and parses it to obtain information about
        # the object's children.  It is a hack but has worked well for
        # a *very* long time with MayaVi-1.x and before.

        # Oops, this isn't a VTK object.
        if not hasattr(vtk_obj, 'GetClassName'):
            return []

        methods = str(vtk_obj)
        methods = methods.split("\n")
        del methods[0]

        # using only the first set of indented values.
        patn = re.compile(r"  \S")
        for method in methods[:]:
            if patn.match(method):
                if method.find(":") == -1:
                    methods.remove(method)
                elif method[1].find("none") > -1:
                    methods.remove(method)
            else:
                methods.remove(method)

        # Props/Prop is deprecated in more recent VTK releases.
        for method in methods[:]:
            if method.strip()[:6] == "Props:":
                if hasattr(vtk_obj, "GetViewProps"):
                    methods.remove(method)
                    methods.append("ViewProps: ")
            elif method.strip()[:5] == "Prop:":
                if hasattr(vtk_obj, "GetViewProp"):
                    methods.remove(method)
                    methods.append("ViewProp: ")

        method_names = []
        for i in range(0, len(methods)):
            strng = methods[i].replace(" ", "")
            methods[i] = strng.split(":")
            method_names.append(methods[i][0])

        if re.match(r"vtk\w*Renderer", vtk_obj.GetClassName()):
            methods.append(["ActiveCamera", ""])

        if re.match(r"vtk\w*Assembly", vtk_obj.GetClassName()):
            methods.append(["Parts", ""])
            methods.append(["Volumes", ""])
            methods.append(["Actors", ""])

        if vtk_obj.IsA('vtkAbstractTransform'):
            if self.last_transform > 0:
                _remove_method('Inverse', methods, method_names)
            else:
                self.last_transform += 1
        else:
            self.last_transform = 0

        # Some of these object are removed because they arent useful in
        # the browser.  I check for Source and Input anyway so I dont need
        # them.
        for name in('Output', 'FieldData', 'CellData', 'PointData',
                    'Source', 'Input', 'ExtentTranslator', 'ProgressText',
                    'Interactor', 'Lights', 'Information', 'Executive'):
            _remove_method(name, methods, method_names)

        return methods


######################################################################
# `CompositeIterable` class.
######################################################################
class CompositeIterable(HasTraits):

    """This class allows one to iterate over a bunch of disparate
    objects treating them as one single iterable.  Each of the
    iterated objects is wrapped with a suitable Node class so that the
    object may be used in a Traits Tree Editor.
    """

    tree_generator = Instance(TreeGenerator)

    def __init__(self, args, **traits):
        super(CompositeIterable, self).__init__(**traits)
        self.args = args

    def __iter__(self):
        tg = self.tree_generator
        for arg in self.args:
            if is_iterable(arg):
                for x in arg:
                    yield tg.get_node(x)
            else:
                yield tg.get_node(arg)

    def __len__(self):
        x = 0
        for arg in self.args:
            if is_iterable(arg):
                x += len(arg)
            else:
                x += 1
        return x


######################################################################
# `TVTKLeafNode` class.
######################################################################
class TVTKLeafNode(TreeNodeObject):
    """Represents a leaf in the tree view."""

    # The tvtk object being wrapped.
    object = Instance(TVTKBase)
    # Name to show on the view.
    name = Property

    # Work around problem with HasPrivateTraits.
    __ = Python

    def __hash__(self):
        return hash(tvtk.to_vtk(self.object))

    def _get_name(self):
        return self.object.__class__.__name__

    ######################################################################
    # `TreeNodeObject` interface
    ######################################################################
    def tno_get_icon(self, node, is_expanded):
        """ Returns the icon for a specified object.
        """
        icon = get_icon(self.name)
        if icon:
            return icon
        else:
            return super(TVTKLeafNode, self).tno_get_icon(node, is_expanded)


######################################################################
# `TVTKBranchNode` class.
######################################################################
class TVTKBranchNode(TreeNodeObject):
    """Represents a branch in the tree view.  The `children` trait
    produces an iterable that represents the children of the branch.
    """
    # The tvtk object being wrapped.
    object = Instance(TVTKBase)
    # Children of the object.
    children = Property
    # Name to show on the view.
    name = Property
    # Tree generator to use.
    tree_generator = Instance(TreeGenerator)
    # Cache of children.
    children_cache = Dict

    # Work around problem with HasPrivateTraits.
    __ = Python

    def __init__(self, **traits):
        super(TVTKBranchNode, self).__init__(**traits)

    def __del__(self):
        try:
            self._remove_listners()
        except:
            pass

    def __hash__(self):
        return hash(tvtk.to_vtk(self.object))

    def _get_children_from_cache(self):
        return [x for x in self.children_cache.values() if x is not None]

    def _create_children(self):
        kids = self.tree_generator.get_children(self.object)
        self.children_cache = kids
        self._setup_listners()

    def _setup_listners(self):
        object = self.object
        kids = self.children_cache
        for key, val in kids.items():
            if isinstance(val, tvtk.Collection):
                vtk_obj = tvtk.to_vtk(val)
                messenger.connect(vtk_obj, 'ModifiedEvent',
                                  self._notify_children)
            else:
                object.on_trait_change(self._notify_children, key)

    def _remove_listners(self):
        object = self.object
        kids = self.children_cache
        for key, val in kids.items():
            if isinstance(val, tvtk.Collection):
                vtk_obj = tvtk.to_vtk(val)
                messenger.disconnect(vtk_obj, 'ModifiedEvent',
                                     self._notify_children)
            else:
                object.on_trait_change(self._notify_children, key, remove=True)

    def _notify_children(self, obj=None, name=None, old=None, new=None):
        old_val = self._get_children_from_cache()
        self._remove_listners()
        self._create_children()
        new_val = self._get_children_from_cache()
        self.trait_property_changed('children', old_val, new_val)

    def _get_children(self):
        if not self.children_cache:
            self._create_children()
        kids = self._get_children_from_cache()
        tg = self.tree_generator
        return CompositeIterable(kids, tree_generator=tg)

    def _get_name(self):
        return self.object.__class__.__name__

    ######################################################################
    # `TreeNodeObject` interface
    ######################################################################
    def tno_get_icon(self, node, is_expanded):
        """ Returns the icon for a specified object.
        """
        icon = get_icon(self.name)
        if icon:
            return icon
        else:
            return super(TVTKBranchNode, self).tno_get_icon(node, is_expanded)


######################################################################
# `TVTKCollectionNode` class.
######################################################################
class TVTKCollectionNode(TreeNodeObject):
    """Represents a collection of typically unconnected roots in the
    tree view.
    """
    # List of child nodes.
    object = List(TVTKBase)
    # Children of the object.
    children = Property
    # Name to show on the view.
    name = Str
    # Tree generator to use.
    tree_generator = Instance(TreeGenerator)

    # Work around problem with HasPrivateTraits.
    __ = Python

    def __init__(self, **traits):
        super(TVTKCollectionNode, self).__init__(**traits)

    def _get_children(self):
        tg = self.tree_generator
        return CompositeIterable(self.object, tree_generator=tg)


######################################################################
# `CloseHandler` class.
######################################################################
class UICloseHandler(TVTKBaseHandler):
    """This class cleans up after the UI for the object is closed."""
    # The browser associated with this UI.
    browser = Any

    def close(self, info, is_ok):
        """This method is invoked when the user closes the UI."""
        obj = info.object
        obj.on_trait_change(self.browser.render, remove=True)
        return True


######################################################################
# `PipelineBrowser` class.
######################################################################
class PipelineBrowser(HasTraits):
    # The tree generator to use.
    tree_generator = Trait(FullTreeGenerator(),
                           Instance(TreeGenerator))

    # The TVTK render window(s) associated with this browser.
    renwins = List

    # The root object to view in the pipeline.  If None (default), the
    # root object is the render_window of the Scene instance passed at
    # object instantiation time.
    root_object = List(TVTKBase)

    selected = Instance(TVTKBase)

    # This is fired when an object has been changed on the UI. Use this when
    # you do not set the renwins list with the render windows but wish to do
    # your own thing when the object traits are edited on the UI.
    object_edited = Event

    # Private traits.
    # The root of the tree to display.
    _root = Any
    _ui = Any

    ###########################################################################
    # `object` interface.
    ###########################################################################
    def __init__(self, renwin=None, **traits):
        """Initializes the object.

        Parameters
        ----------

        - renwin: `Scene` instance.  Defaults to None.

          This may be passed in addition to the renwins attribute
          which can be a list of scenes.

        """
        super(PipelineBrowser, self).__init__(**traits)
        self._ui = None
        self.view = None
        if renwin:
            self.renwins.append(renwin)

        self._root_object_changed(self.root_object)

    def default_traits_view(self):
        menu = Menu(Action(name='Refresh', action='editor.update_editor'),
                    Action(name='Expand all', action='editor.expand_all'))
        self.menu = menu

        nodes = self.tree_generator.get_nodes(menu)

        self.tree_editor = TreeEditor(nodes=nodes,
                                      editable=False,
                                      orientation='vertical',
                                      hide_root=True,
                                      on_select=self._on_select,
                                      on_dclick=self._on_dclick)
        view = View(Group(VSplit(Item(name='_root',
                                      editor=self.tree_editor,
                                      resizable=True),
                                 Item(name='selected',
                                      style='custom',
                                      resizable=True),
                                 show_labels=False,
                                 show_border=False)),
                    title='Pipeline browser',
                    help=False,
                    resizable=True, undo=False, revert=False,
                    width=.3, height=.3)
        return view

    ###########################################################################
    # `PipelineBrowser` interface.
    ###########################################################################
    def show(self, parent=None):
        """Show the tree view if not already show.  If optional
        `parent` widget is passed, the tree is displayed inside the
        passed parent widget."""
        # If UI already exists, raise it and return.
        if self._ui is not None and self._ui.control is not None:
            try:
                self._ui.control.Raise()
            except AttributeError:
                pass
            else:
                return
        else:
            # No active ui, create one.
            view = self.default_traits_view()
            if parent is not None:
                self._ui = view.ui(self, parent=parent, kind='subpanel')
            else:
                self._ui = view.ui(self, parent=parent)

    def update(self):
        """Update the tree view."""
        # This is a hack.
        if self._ui is not None and self._ui.control is not None:
            try:
                ed = self._ui._editors[0]
                ed.update_editor()
                self._ui.control.Refresh()
            except (AttributeError, IndexError):
                pass
    # Another name for update.
    refresh = update

    def render(self):
        """Calls render on all render windows associated with this
        browser."""
        self.object_edited = True
        for rw in self.renwins:
            rw.render()

    ###########################################################################
    # Non-public interface.
    ###########################################################################
    def _make_default_root(self):
        tree_gen = self.tree_generator
        objs = [x.render_window for x in self.renwins]
        node = TVTKCollectionNode(object=objs, name="Root",
                                  tree_generator=tree_gen)
        return node

    def _tree_generator_changed(self, tree_gen):
        """Traits event handler."""
        if self._root:
            root_obj = self._root.object
        else:
            root_obj = self.root_object
        if root_obj:
            ro = root_obj
            if not hasattr(root_obj, '__len__'):
                ro = [root_obj]

            self._root = TVTKCollectionNode(object=ro,
                                            name="Root",
                                            tree_generator=tree_gen)
        else:
            self._root = self._make_default_root()

        self.tree_editor.nodes = tree_gen.get_nodes(self.menu)
        self.update()

    def _root_object_changed(self, root_obj):
        """Trait handler called when the root object is assigned to."""
        tg = self.tree_generator
        if root_obj:
            self._root = TVTKCollectionNode(object=root_obj, name="Root",
                                            tree_generator=tg)
        else:
            self._root = self._make_default_root()
            self.root_object = self._root.object
        self.update()

    def _root_object_items_changed(self, list_event):
        """Trait handler called when the items of the list change."""
        self._root_object_changed(self.root_object)

    def _on_dclick(self, obj):
        """Callback that is called when nodes are double-clicked."""
        if hasattr(obj, 'object') and hasattr(obj.object, 'edit_traits'):
            object = obj.object
            view = object.trait_view()
            view.handler = UICloseHandler(browser=self)
            object.on_trait_change(self.render)
            ui = object.edit_traits(view=view)

    def _on_select(self, obj):
        if hasattr(obj, 'object') and hasattr(obj.object, 'edit_traits'):
            new = obj.object
            old = self.selected
            if new != old:
                self.selected = new
            if old is not None:
                old.on_trait_change(self.render, remove=True)
            if new is not None:
                new.on_trait_change(self.render)



######################################################################
# Test cases.
######################################################################
def main(instantiate_gui=True):
    """Simple test case."""
    from tvtk.tools import ivtk

    v = ivtk.viewer(browser=False, instantiate_gui=instantiate_gui)
    cs = tvtk.ConeSource()
    m = tvtk.PolyDataMapper(input_connection=cs.output_port)
    a = tvtk.Actor(mapper=m)
    v.scene.add_actor(a)
    v.scene.reset_zoom()

    b = PipelineBrowser(v.scene)
    b.show()

    return v, b, a


if __name__ == '__main__':
    from pyface.api import GUI
    gui = GUI()
    v, b, a = main(instantiate_gui=False)
    gui.start_event_loop()



"""The Mayavi plugin.
"""
# Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
# Copyright (c) 2008,  Enthought, Inc.
# License: BSD Style.

from traits.api import List
from envisage.api import Plugin, ServiceOffer

# This module's package.
PKG = '.'.join(__name__.split('.')[:-1])
# The mayavi package ID.
ID = 'mayavi'

###############################################################################
# `MayaviPlugin` class.
###############################################################################
class MayaviPlugin(Plugin):

    # Extension point Ids.
    SERVICE_OFFERS = 'envisage.ui.workbench.service_offers'
    PREFERENCES       = 'envisage.preferences'

    # The plugins name.
    name = 'Mayavi plugin'

    # Our ID.
    id = ID

    ###### Contributions to extension points made by this plugin ######

    # Services we contribute.
    service_offers = List(contributes_to=SERVICE_OFFERS)

    # Preferences.
    preferences = List(contributes_to=PREFERENCES)

    def _preferences_default(self):
        """ Trait initializer. """
        return ['pkgfile://%s/preferences/preferences.ini' % ID]


    ######################################################################
    # Private methods.
    def _service_offers_default(self):
        """ Trait initializer. """
        engine_service_offer = ServiceOffer(
            protocol = 'mayavi.core.engine.Engine',
            factory  = PKG + '.envisage_engine.EnvisageEngine'
        )

        script_service_offer = ServiceOffer(
            protocol = 'mayavi.plugins.script.Script',
            factory  = PKG + '.script.Script'
        )
        return [engine_service_offer, script_service_offer]




"""
Traits View definition file.

The view trait of the parent class has been extracted from the model
definition file.  This file can either be exec()ed or imported.  See
core/base.py:Base.trait_view() for what is currently used.  Using exec()
allows view changes without needing to restart Mayavi, but is slower than
importing.
"""
# Authors: Prabhu Ramachandran <prabhu_r@users.sf.net>
#          Judah De Paula <judah@enthought.com>
# Copyright (c) 2005-2008, Enthought, Inc.
# License: BSD Style.

from traitsui.api import Item, Group, View

view = View(Group(Item(name='filled_contours',
                       defined_when='show_filled_contours'),
                  Item(name='auto_contours'),

                  # One group or the other, but not both.
                  Group(
                       Item(name='contours',
                            style='custom',
                            visible_when='not auto_contours',
                            show_label=False),
                  ),
                  Group(
                      Item(name='number_of_contours'),
                      Item(name='minimum_contour'),
                      Item(name='maximum_contour'),
                      visible_when='auto_contours',
                  ),

                  Item(name='auto_update_range'),
                  Group(
                        Item(name='_data_min',
                             label='Data minimum'),
                        Item(name='_data_max',
                             label='Data maximum'),
                             visible_when='not auto_update_range',
                  )
               )
           )



"""An importer for 3D Studio files.

"""
# Author: Prabhu Ramachandran <prabhu at aero dot iitb dot ac dot in>
# Copyright (c) 2007, Enthought, Inc.
# License: BSD Style.

# Standard library imports.
from os.path import basename

# Enthought imports.
from tvtk.api import tvtk
from traits.api import Instance

# Local imports
from mayavi.sources.vrml_importer import VRMLImporter


######################################################################
# `ThreeDSImporter` class.
######################################################################
class ThreeDSImporter(VRMLImporter):

    # The 3DS importer.
    reader = Instance(tvtk.ThreeDSImporter, args=(),
                      kw={'compute_normals':True},
                      allow_none=False, record=True)

    ######################################################################
    # `FileDataSource` interface
    ######################################################################

    def has_output_port(self):
        """ Return True as the reader has output port."""
        return True

    def get_output_object(self):
        """ Return the reader output port."""
        return self.reader.output_port
    
    ######################################################################
    # Non-public interface
    ######################################################################
    def _file_name_changed(self, value):
        # This hack is necessary since for some reason the importer
        # does not clear out the earlier actors.
        self.reader = reader = tvtk.ThreeDSImporter(compute_normals=True)
        reader.file_name = value
        if self.scene is not None:
            self.reader.render_window = self.scene.render_window

        name = "3DStudio file (%s)"%basename(self.file_name)
        if '[Hidden]' in self.name:
            name += ' [Hidden]'
        self.name = name

        self._file_path.trait_set(value)
        self._update_reader()
        self.render()



"""
Common code for mayavi tests.

"""
# Author: Prabhu Ramachandran <prabhu@aero.iitb.ac.in>
# Copyright (c) 2008, Enthought, Inc.
# License: BSD Style.
import os.path

from traits.api import HasTraits, Any, Event, Callable

def fixpath(filename):
    """Given a relative file path it sets the path relative to this
    directory.  This allows us to run the tests from other directories
    as well.
    """
    return os.path.join(os.path.dirname(__file__), filename)


def get_example_data(fname):
    """Given a relative path to data inside the examples directory,
    obtains the full path to the file.
    """
    p = os.path.join('data', fname)
    return os.path.abspath(fixpath(p))




import unittest
import numpy as np

from tvtk.api import tvtk
from mayavi.core.utils import DataSetHelper
from vtk.numpy_interface import dataset_adapter as dsa


class TestDataSetHelper(unittest.TestCase):
    def _make_data(self, factor=1.0):
        id = tvtk.ImageData(dimensions=(2, 2, 1), origin=(0, 0, 0),
                            spacing=(1, 1, 1))
        id.point_data.scalars = np.arange(4, dtype=float)*factor
        id.point_data.scalars.name = 'ps'
        id.point_data.vectors = np.ones((4, 3), dtype=float)*factor
        id.point_data.vectors.name = 'pv'
        id.cell_data.scalars = np.arange(4, dtype=float)*0.5*factor
        id.cell_data.scalars.name = 'cs'
        id.cell_data.vectors = np.ones((4, 3), dtype=float)*0.5*factor
        id.cell_data.vectors.name = 'cv'
        return id

    def test_get_range_works_for_simple_datasets(self):
        # Given
        id = self._make_data()

        # When
        dsh = DataSetHelper(id)

        # Then
        name, rng = dsh.get_range('scalars', 'point')
        self.assertEqual(name, 'ps')
        self.assertEqual(rng, [0.0, 3.0])

        name, rng = dsh.get_range('scalars', 'cell')
        self.assertEqual(name, 'cs')
        self.assertEqual(rng, [0.0, 1.5])

        name, rng = dsh.get_range('vectors', 'point')
        self.assertEqual(name, 'pv')
        self.assertEqual(rng, [0.0, np.sqrt(3.0)])

        name, rng = dsh.get_range('vectors', 'cell')
        self.assertEqual(name, 'cv')
        self.assertEqual(rng, [0.0, np.sqrt(3.0)*0.5])

    def test_get_range_works_for_multiblock_datasets(self):
        # Given
        id1 = self._make_data()
        id2 = self._make_data(factor=2.0)

        mb = tvtk.MultiBlockDataSet()
        mb.set_block(0, id1)
        mb.set_block(1, id2)

        # When
        dsh = DataSetHelper(mb)

        # Then
        name, rng = dsh.get_range('scalars', 'point')
        self.assertEqual(name, 'ps')
        self.assertEqual(rng, [0.0, 6.0])

        name, rng = dsh.get_range('scalars', 'cell')
        self.assertEqual(name, 'cs')
        self.assertEqual(rng, [0.0, 3.0])

        name, rng = dsh.get_range('vectors', 'point')
        self.assertEqual(name, 'pv')
        self.assertEqual(rng, [0.0, np.sqrt(3.0)*2.0])

        name, rng = dsh.get_range('vectors', 'cell')
        self.assertEqual(name, 'cv')
        self.assertEqual(rng, [0.0, np.sqrt(3.0)])

    def test_get_range_works_for_VTKNoneArray(self):
        id_ = tvtk.ImageData(dimensions=(2, 2, 1), origin=(0, 0, 0),
                             spacing=(1, 1, 1))
        id_.point_data.scalars = np.arange(4, dtype=float)
        dsh = DataSetHelper(id_)
        self.assertFalse(dsh._composite)
        self.assertEqual(dsh.get_range(), (None, [0., 1.]))
        # XXX there is some wackiness here, no idea why this changes!
        self.assertEqual(dsh.get_range(), ('point_scalars', [0., 3.]))

if __name__ == '__main__':
    unittest.main()



# Author: Gael Varoquaux <gael _dot_ varoquaux _at_ normalesup _dot_ org>
# Copyright (c) 2008, Enthought, Inc.
# License: BSD Style.

# Local imports
from mayavi.filters.filter_base import FilterBase
from mayavi.components.common import convert_to_poly_data


######################################################################
# `PolyDataFilterBase` class.
######################################################################
class PolyDataFilterBase(FilterBase):

    """ Base class for a filter requiring polydata input. Converts the
        source to polydata.
    """

    ######################################################################
    # `Filter` interface.
    ######################################################################
    def update_pipeline(self):
        # Do nothing if there is no input.
        inputs = self.inputs
        if len(inputs) == 0:
            return

        # By default we set the input to the first output of the first
        # input.
        fil = self.filter
        self.configure_input(fil, convert_to_poly_data(inputs[0].outputs[0]))
        fil.update()
        self._set_outputs([fil])



# Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
# Copyright (c) 2008,  Prabhu Ramachandran
# License: BSD Style.

# Enthought library imports.
from traits.api import HasTraits, List, Str, Instance
from apptools.preferences.api import PreferencesHelper


###############################################################################
# `PreferencesMirror` class.
###############################################################################
class PreferencesMirror(HasTraits):
    """
    This class mirrors preferences from a PreferencesHelper such that
    users can use them and change them but nothing is saved to disk till
    the user asks for an explicit save.
    """

    # The preferences we mirror.
    preferences = Instance(PreferencesHelper)

    # Private trait to store names of traits.
    _trait_names = List(Str)

    ######################################################################
    # Public interface.
    ######################################################################
    def save(self):
        """Updates the actual preferences and thereby persists them to
        disk.
        """
        for name in self._trait_names:
            setattr(self.preferences, name, getattr(self, name))

    ######################################################################
    # Private interface.
    ######################################################################
    def _preferences_changed(self):
        """Setup traits of our own based on those of the mayavi
        preferences.
        """
        trait_names = []
        opts = self.preferences
        for key, value in opts.traits().items():
            if key not in ['trait_added', 'trait_modified',
                           'preferences', 'preferences_path']:
                self.add_trait(key, value)
                setattr(self, key, getattr(opts, key))
                trait_names.append(key)
                opts.on_trait_change(self._update, key)
        self._trait_names = trait_names

    def _update(self, obj, name, old, new):
        setattr(self, name, new)



from mayavi.tools.mlab_scene_model import MlabSceneModel
from mayavi.core.ui.mayavi_scene import MayaviScene, \
                    DecoratedScene
from tvtk.pyface.scene_editor import SceneEditor
from mayavi.core.ui.engine_view import EngineView
from mayavi.core.ui.engine_rich_view import EngineRichView




import unittest

import numpy as np
from numpy.testing import assert_allclose

from tvtk.tools import visual

def get_bounds(pos, sz):
    pos = np.asarray(pos)
    sz = np.asarray(sz)
    diff = sz*0.5
    bounds = np.zeros(6, dtype=float)
    bounds[::2] = pos - diff
    bounds[1::2] = pos + diff
    return bounds

class DummyScene:
    def __init__(self):
        self.actors = []

    def add_actors(self, *args):
        self.actors.extend(args)

    def remove_actors(self, *args):
        for arg in args:
            self.actors.remove(arg)

    def render(self):
        pass

    def reset_zoom(self):
        pass


class DummyViewer:
    def __init__(self):
        self.scene = DummyScene()


class TestVisual(unittest.TestCase):
    def setUp(self):
        self.viewer = DummyViewer()
        visual.set_viewer(self.viewer)

    def tearDown(self):
        visual.set_viewer(None)

    def test_ring(self):
        # Given
        r = visual.ring()

        # Then
        assert_allclose(r.polydata.bounds, (0.0, 0.0, -0.5, 0.5, -0.5, 0.5))
        self.assertEqual(r.radius, 0.5)
        self.assertEqual(r.tube.radius, 0.01)

        # Given
        r = visual.ring(radius=1.0, pos=(1.0, 1.0, 1.0))

        # When/Then
        self.assertEqual(r.radius, 1.0)
        self.assertEqual(r.thickness, 0.01)
        self.assertEqual(r.tube.radius, 0.01)
        assert_allclose(r.pos, (1.0, 1.0, 1.0))
        assert_allclose(r.polydata.bounds, (1.0, 1.0, 0.0, 2.0, 0.0, 2.0))

        # When
        r.x = 10.0
        # Then
        assert_allclose(r.pos, (10.0, 1.0, 1.0))

        # When
        r = visual.ring(axis=(0., 1., 0.))
        # Then
        assert_allclose(r.polydata.bounds, (-0.5, 0.5, 0.0, 0.0, -0.5, 0.5),
                        atol=1e-8, rtol=0)
        self.assertEqual(r.radius, 0.5)
        self.assertEqual(r.tube.radius, 0.01)

        # When
        r.axis = 1,0,0
        assert_allclose(r.polydata.bounds, (0.0, 0.0, -0.5, 0.5, -0.5, 0.5))

    def test_cone(self):
        # Given
        c = visual.cone()

        # Then
        z = np.sqrt(3)/4
        assert_allclose(c.polydata.bounds,
                        (-0.5, 0.5, -0.5, 0.5, -z, z))
        self.assertEqual(c.radius, 0.5)
        self.assertEqual(c.height, 1.0)

        # When
        c = visual.cone(pos=(1.0, 1.0, 1.0))

        # Then
        z = np.sqrt(3)/4
        assert_allclose(c.polydata.bounds,
                        (0.5, 1.5, 0.5, 1.5, 1-z, 1+z))

        # When
        c = visual.cone(axis=(0, 1, 0), height=2.0)
        # Then
        z = np.sqrt(3)/4
        assert_allclose(c.polydata.bounds,
                        (-0.5, 0.5, -1, 1., -z, z))

    def test_sphere(self):
        # When
        s = visual.sphere()

        # Then

        # The discrete representation of the sphere makes the bounds a bit
        # smaller than one would expect.
        assert_allclose(s.polydata.bounds,
                        (-0.5, 0.5, -0.5, 0.5, -0.5, 0.5), rtol=1e-2)
        self.assertEqual(s.radius, 0.5)

        # When
        s = visual.sphere(pos=(1.0, 1.0, 1.0))
        # Then
        assert_allclose(s.polydata.bounds,
                        (0.5, 1.5, 0.5, 1.5, 0.5, 1.5), rtol=1e-2)

        # When
        s = visual.sphere(x=1.0)
        # Then
        assert_allclose(s.polydata.bounds,
                        (0.5, 1.5, -0.5, 0.5, -0.5, 0.5), rtol=1e-2)

    def test_cylinder(self):
        # When
        c = visual.cylinder(radius=0.5, length=2, pos=(1., 1, 1))

        # Then
        assert_allclose(
            c.polydata.bounds, (1.0, 3.0, 0.5, 1.5, 0.5, 1.5), rtol=1e-2
         )

        # When
        c = visual.cylinder(radius=0.5, length=2, x=1.0)

        # Then
        assert_allclose(
            c.polydata.bounds, (1.0, 3.0, -0.5, 0.5, -0.5, 0.5), rtol=5e-2
         )

    def test_box(self):
        # When
        pos = np.array((1.0, 2.0, 3.0))
        sz = pos.copy()
        b = visual.box(size=tuple(sz), pos=pos)

        # Then
        bounds = get_bounds(pos, sz)
        assert_allclose(b.polydata.bounds, bounds)

        # Given
        b = visual.box()
        orig_bounds = (-0.5, 0.5, -0.5, 0.5, -0.5, 0.5)
        assert_allclose(b.polydata.bounds, orig_bounds)
        # When
        b.axis = 1,1,0
        # Then
        x = np.sqrt(2)
        bounds = get_bounds((0.,0,0), (x, x, 1.0))
        assert_allclose(b.polydata.bounds, bounds)

        # When
        b.axis = 1,0,0
        assert_allclose(b.polydata.bounds, orig_bounds)

    def test_arrow(self):
        # Given
        a = visual.arrow()
        # Then
        self.assertEqual(a.radius_cone, 0.08)
        self.assertEqual(a.radius_shaft, 0.03)
        self.assertEqual(a.length_cone, 0.35)
        bounds = get_bounds((0.5, 0.0, 0.0), (1.0, 0.16, 0.14))
        assert_allclose(a.polydata.bounds, bounds, atol=1e-3, rtol=0)

        # When
        a.axis = 0, 0, 1
        # Then
        bounds = get_bounds((0.0, 0.0, 0.5), (0.14, 0.16, 1.0))
        assert_allclose(a.polydata.bounds, bounds, atol=1e-3, rtol=0)

        # Given
        a = visual.arrow(pos=(1.0, 1.0, 1.0))
        # Then
        bounds = get_bounds((1.5, 1.0, 1.0), (1.0, 0.16, 0.14))
        assert_allclose(a.pos, (1., 1., 1.))
        assert_allclose(a.polydata.bounds, bounds, atol=1e-3, rtol=0)

    def test_ellipsoid(self):
        # When
        s = visual.ellipsoid(size=(1.0, 1.0, 1.0), pos=(1., 1., 1.))

        # Then

        # The discrete representation of the sphere makes the bounds a bit
        # smaller than one would expect.
        bounds = get_bounds((1., 1, 1), (1., 1, 1))
        assert_allclose(s.polydata.bounds, bounds, rtol=1e-2)
        self.assertEqual(s.radius, 0.5)

        # Given/When
        s = visual.ellipsoid(size=(1.0, 1.0, 0.5))

        # Then
        bounds = get_bounds((0., 0, 0), (1., 1.0, 0.5))
        assert_allclose(s.polydata.bounds, bounds, rtol=1e-2)
        self.assertEqual(s.length, 1.0)
        self.assertEqual(s.height, 1.0)
        self.assertEqual(s.width, 0.5)

        # Given/When
        s = visual.ellipsoid(length=1.0, height=1.0, width=0.5)

        # Then
        bounds = get_bounds((0., 0, 0), (1., 1.0, 0.5))
        assert_allclose(s.polydata.bounds, bounds, rtol=1e-2)
        assert_allclose(s.size, (1.0, 1.0, 0.5))
        self.assertEqual(s.length, 1.0)
        self.assertEqual(s.height, 1.0)
        self.assertEqual(s.width, 0.5)

    def test_curve(self):
        # Given/When
        c = visual.curve(points=[[0.,0.,0.],[1.,1.,1.]], pos=(1.,1.,1.))

        # Then
        bounds = get_bounds((1.5, 1.5, 1.5), (1.0, 1.0, 1.0))
        assert_allclose(c.polydata.bounds, bounds)

        # When
        c = visual.curve(points=[[0.,0,0],[1.,1,1]], axis=(0., 1, 0))
        # Then
        bounds = get_bounds((-0.5, 0.5, 0.5), (1.0, 1.0, 1.0))
        assert_allclose(c.polydata.bounds, bounds)

    def test_helix(self):
        # Given/When
        h = visual.helix(pos=(1., 1.,1.))

        # Then
        bounds = get_bounds((1.5, 1.0, 1.0), (1.0, 0.4, 0.4))
        assert_allclose(h.polydata.bounds, bounds, atol=3e-2, rtol=0)

        # Given/When
        h.axis = 0., 1., 0.
        # Then
        bounds = get_bounds((1.0, 1.5, 1.0), (0.4, 1.0, 0.4))
        assert_allclose(h.polydata.bounds, bounds, atol=3e-2, rtol=0)

        # Given/When
        h.axis = 1, 0, 0
        # Then
        bounds = get_bounds((1.5, 1.0, 1.0), (1.0, 0.4, 0.4))
        assert_allclose(h.polydata.bounds, bounds, atol=3e-2, rtol=0)

    def test_frame(self):
        # Given
        s = visual.sphere()
        a = visual.arrow()
        f = visual.frame(s, a)

        # Then
        axis = (1.0, 0, 0)
        assert_allclose(s.axis, axis, rtol=0, atol=1e-15)
        assert_allclose(a.axis, axis, rtol=0, atol=1e-15)
        assert_allclose(f.axis, axis, rtol=0, atol=1e-15)

        # When
        axis = (0, 1.0, 0.0)
        f.axis = axis

        # Then
        assert_allclose(s.axis, axis, rtol=0, atol=1e-15)
        assert_allclose(a.axis, axis, rtol=0, atol=1e-15)
        assert_allclose(f.axis, axis, rtol=0, atol=1e-15)

        # When
        pos = (1., 1.0, 1.0)
        f.pos = pos

        # Then
        assert_allclose(s.pos, pos, rtol=0, atol=1e-15)
        assert_allclose(a.pos, pos, rtol=0, atol=1e-15)
        assert_allclose(f.pos, pos, rtol=0, atol=1e-15)

if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2015,  Enthought, Inc.
# License: BSD Style.

# Standard library imports.
import unittest

# External library imports
import vtk

# Enthought library imports
from mayavi.sources.vtk_file_reader import VTKFileReader

# Local imports.
from mayavi.tests.common import get_example_data

vtk_major_version = vtk.vtkVersion.GetVTKMajorVersion()
vtk_minor_version = vtk.vtkVersion.GetVTKMinorVersion()

class TestVTKFileReader(unittest.TestCase):
    def setUp(self):
        # Read a VTK data file.
        self.src = VTKFileReader()

    def tearDown(self):
        """For necessary clean up, automatically called by TestCase
           after the test methods have been invoked"""
        self.src = None
        return

    def check(self, n_points, n_cells):
        """Do the actual testing."""
        output = self.src.get_output_dataset()
        self.assertEqual(output.number_of_points, n_points)
        self.assertEqual(output.number_of_cells, n_cells)

    def test_structured_points_file(self):
        self.src.initialize(get_example_data('texThres2.vtk'))
        self.check(128, 127)

    def test_rectiliner_grid_file(self):
        self.src.initialize(get_example_data('RectGrid2.vtk'))
        self.check(17061, 14720)

    def test_polydata_file(self):
        self.src.initialize(get_example_data('polyEx.vtk'))
        self.check(8, 6)

    def test_structured_grid_file(self):
        self.src.initialize(get_example_data('SampleStructGrid.vtk'))
        self.check(24000, 21489)

    def test_unstructured_grid_file(self):
        self.src.initialize(get_example_data('uGridEx.vtk'))
        self.check(27, 12)

    def test_field_file(self):
        self.src.initialize(get_example_data('fieldfile.vtk'))
        self.check(18, 3)

if __name__ == '__main__':
    unittest.main()



"""
HasTraits class providing window with a mayavi engine, to preview pipeline
elements.
"""

from traits.api import HasTraits, Instance
from traitsui.api import View, Item
from tvtk.pyface.scene_editor import SceneEditor
from tvtk.pyface.scene_model import SceneModel
from tvtk.pyface.scene import Scene

from mayavi.core.engine import Engine


##############################################################################
# PreviewWindow class
##############################################################################
class PreviewWindow(HasTraits):
    """ A window with a mayavi engine, to preview pipeline elements.
    """

    # The engine that manages the preview view
    _engine = Instance(Engine)

    _scene = Instance(SceneModel, ())

    view = View(Item('_scene', editor=SceneEditor(scene_class=Scene),
                        show_label=False),
                width=500, height=500)

    #-----------------------------------------------------------------------
    # Public API
    #-----------------------------------------------------------------------

    def add_source(self, src):
        self._engine.add_source(src)

    def add_module(self, module):
        self._engine.add_module(module)

    def add_filter(self, filter):
        self._engine.add_module(filter)

    def clear(self):
        self._engine.current_scene.scene.disable_render = True
        self._engine.current_scene.children[:] = []
        self._engine.current_scene.scene.disable_render = False

    #-----------------------------------------------------------------------
    # Private API
    #-----------------------------------------------------------------------

    def __engine_default(self):
        e = Engine()
        e.start()
        e.new_scene(self._scene)
        return e

if __name__ == '__main__':
    from pyface.api import GUI
    from mayavi.sources.api import ParametricSurface
    from mayavi.modules.api import Outline, Surface
    pw = PreviewWindow()
    pw.add_source(ParametricSurface())
    pw.add_module(Outline())
    pw.add_module(Surface())

    pw.edit_traits()



"""Mayavi action set for menus and actions etc.
"""
# Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
# Copyright (c) 2008, Enthought, Inc.
# License: BSD Style.

# Enthought library imports.
from envisage.ui.action.api import Action, ActionSet, Group, Menu
from mayavi.core.registry import registry

########################################
# Groups

file_group = Group(
    id='MayaviFileGroup',
    path='MenuBar/File',
    before='ExitGroup'
)

visualize_group = Group(
    id='VisualizeGroup',
    path='MenuBar/VisualizeMenu',
)

modules_group = Group(
    id='ModulesGroup',
    path='MenuBar/VisualizeMenu/ModulesMenu',
)

filters_group = Group(
    id='FiltersGroup',
    path='MenuBar/VisualizeMenu/FiltersMenu',
)
########################################
# Menus

open_menu = Menu(
    id     = "LoadDataMenu",
    name   = "&Load data",
    path = 'MenuBar/File',
    group='MayaviFileGroup'
)

visualize_menu = Menu(
    id     = "VisualizeMenu",
    name   = "Visuali&ze",
    path = "MenuBar",
    before = "View"
)

modules_menu = Menu(
    id     = "ModulesMenu",
    name   = "&Modules",
    path="MenuBar/VisualizeMenu",
)

filters_menu = Menu(
    id     = "FiltersMenu",
    name   = "&Filters",
    path="MenuBar/VisualizeMenu",
    after="ModulesMenu",
)

########################################
# File menu items.

ID = 'mayavi'

####################
# Source actions.

open_file = Action(
    id            = "OpenFile",
    class_name    = ID + ".action.sources.OpenFile",
    name          = "&Open file ...",
    path        =  "MenuBar/File/LoadDataMenu"
)

# Automatic source generation for non-open file related sources.
SOURCE_ACTIONS = [open_file]

for src in registry.sources:
    if len(src.extensions) == 0:
        action = Action(id=src.id,
                        class_name=ID + ".action.sources." + src.id,
                        name= src.menu_name,
                        path="MenuBar/File/LoadDataMenu"
                        )
        SOURCE_ACTIONS.append(action)

####################
# Save/load actions.

save_viz = Action(
    id            = "SaveVisualization",
    class_name    = ID + ".action.save_load.SaveVisualization",
    name          = "&Save Visualization",
    group = "MayaviFileGroup",
    path="MenuBar/File",
)

load_viz = Action(
    id            = "LoadVisualization",
    class_name    = ID + ".action.save_load.LoadVisualization",
    name          = "&Load Visualization",
    group = "MayaviFileGroup",
    path="MenuBar/File",
)

run_script = Action(
    id            = "RunScript",
    class_name    = ID + ".action.save_load.RunScript",
    name          = "&Run Python Script",
    group = "MayaviFileGroup",
    path="MenuBar/File",
)

########################################
# Visualize menu items.

add_mm = Action(
    id            = "AddModuleManager",
    class_name    = ID + ".action.modules.AddModuleManager",
    name          = "&Add ModuleManager",
    path="MenuBar/VisualizeMenu",
    after="FiltersMenu"
)

########################################
# Modules.

MODULE_ACTIONS = []
for module in registry.modules:
    action = Action(id=module.id,
                    class_name=ID + ".action.modules." + module.id,
                    name= module.menu_name,
                    group = "ModulesGroup",
                    path="MenuBar/VisualizeMenu/ModulesMenu"
                    )
    MODULE_ACTIONS.append(action)

########################################
# Filter items.
########################################
FILTER_ACTIONS = []
for filter in registry.filters:
    action = Action(id=filter.id,
                    class_name=ID + ".action.filters." + filter.id,
                    name= filter.menu_name,
                    group = "FiltersGroup",
                    path="MenuBar/VisualizeMenu/FiltersMenu"
                    )
    FILTER_ACTIONS.append(action)

########################################
# Help menu items.

help_index = Action(
    id            = "HelpIndex",
    class_name    = ID + ".action.help.HelpIndex",
    name          = "&User Guide",
    path="MenuBar/Help"
)


tvtk_class_browser = Action(
    id            = "TVTKClassBrowser",
    class_name    = ID + ".action.help.TVTKClassBrowser",
    name          = "&VTK Class Browser",
    path          = "MenuBar/Help",
)


################################################################################
# `MayaviUIActionSet` class.
################################################################################
class MayaviUIActionSet(ActionSet):
    """ The default action set for the mayavi UI plugin. """

    groups  = [file_group,
               visualize_group,
               modules_group,
               filters_group,
               ]
    menus = [open_menu,
             visualize_menu,
             modules_menu,
             filters_menu
             ]
    actions = SOURCE_ACTIONS + \
              [save_viz, # Save load actions.
               load_viz,
               run_script,
               # Add module manager.
               add_mm,
              ] + \
              MODULE_ACTIONS + \
              FILTER_ACTIONS + \
              [help_index,
               tvtk_class_browser,
              ]
